"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = void 0;
const fs_1 = require("fs");
const path = require("path");
const ts = require("typescript");
const converter_1 = require("./converter/converter");
const serialization_1 = require("./serialization");
const utils_1 = require("./utils");
const renderer_1 = require("./renderer");
const sources_1 = require("./plugins/sources");
const sort_1 = require("./plugins/sort");
class Application {
    /**
     * Create a new TypeDoc application instance.
     *
     * @param options An object containing the options that should be used.
     */
    constructor() {
        /**
         * The converter used to create the declaration reflections.
         */
        this.converter = new converter_1.Converter(this);
        /**
         * The serializer used to generate JSON output.
         */
        this.serializer = new serialization_1.Serializer();
        /**
         * Component used to render reflections into output that is human readable.
         */
        this.renderer = new renderer_1.Renderer(this);
        /**
         * The logger that should be used to output messages.
         */
        this.logger = new utils_1.ConsoleLogger();
        /**
         * Container describing all TypeDoc + TS options.
         */
        this.options = new utils_1.Options(this.logger);
        this.options.addDefaultDeclarations();
    }
    /**
     * Initialize TypeDoc with the given options object.
     *
     * @param options  The desired options to set.
     */
    bootstrap(options = {}) {
        for (const [key, val] of Object.entries(options)) {
            try {
                this.options.setValue(key, val);
            }
            catch (_a) {
                // Ignore errors, plugins haven't been loaded yet and may declare an option.
            }
        }
        this.options.read(new utils_1.Logger());
        const logger = this.options.getValue("logger");
        if (typeof logger === "function") {
            this.logger = new utils_1.CallbackLogger(logger);
            this.options.setLogger(this.logger);
        }
        else if (logger === "none") {
            this.logger = new utils_1.Logger();
            this.options.setLogger(this.logger);
        }
        this.logger.level = this.options.getValue("logLevel");
        sources_1.load(this);
        sort_1.load(this);
        utils_1.loadPlugins(this, this.options.getValue("plugin"), this.logger);
        this.options.reset();
        for (const [key, val] of Object.entries(options)) {
            try {
                this.options.setValue(key, val);
            }
            catch (error) {
                this.logger.error(error.message);
            }
        }
        this.options.read(this.logger);
    }
    /**
     * Run the converter for the given set of files and return the generated reflections.
     *
     * @param src  A list of source that should be compiled and converted.
     * @returns An instance of ProjectReflection on success, undefined otherwise.
     */
    async convert() {
        const entryPoints = await utils_1.expandDirectories(this.options.getValue("entryPoint"), this.options.getValue("exclude"), !!this.options.getCompilerOptions().allowJs);
        const start = Date.now();
        const program = ts.createProgram(entryPoints, this.options.getCompilerOptions());
        const beforeErrors = Date.now();
        this.logger.verbose(`[Perf] Creating program took ${beforeErrors - start}ms`);
        const errors = ts.getPreEmitDiagnostics(program);
        this.logger.verbose(`[Perf] Checking errors took ${Date.now() - beforeErrors}ms`);
        if (errors.length) {
            this.logger.diagnostics(errors);
            return;
        }
        return this.converter.convert(program, entryPoints);
    }
    /**
     * Emit rendered documentation for the given project.
     * @param out the directory to write the rendered documentation to.
     */
    async generateDocs(project, out) {
        out = path.resolve(out);
        await this.renderer.render(project, out);
        this.logger.success("Documentation generated at %s", out);
    }
    /**
     * Run the converter for the given set of files and write the reflections to a json file.
     *
     * @param out The path and file name to write the serialized project to.
     */
    async generateJson(project, out) {
        const serialized = this.serializer.toObject(project);
        await fs_1.promises.writeFile(out, JSON.stringify(serialized, null, "\t"));
        this.logger.success("JSON written to %s", out);
    }
}
exports.Application = Application;
//# sourceMappingURL=application.js.map