"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Converter = void 0;
const assert = require("assert");
const ts = require("typescript");
const path_1 = require("path");
const index_1 = require("../models/index");
const object_1 = require("../models/reflections/object");
const event_1 = require("../utils/event");
const context_1 = require("./context");
const nodes_1 = require("./nodes");
const types_1 = require("./types");
const fs_1 = require("../utils/fs");
const comments_1 = require("./comments");
const utils_1 = require("./utils");
const array_1 = require("../utils/array");
class Converter extends event_1.EventEmitter {
    constructor(application) {
        super();
        this.application = application;
        this._reflectionConverters = new Map();
        this._typeNodeConverters = new Map();
        // HACK: This shouldn't be necessary... but right now recursive inferred types
        // can't be properly dealt with. Used to convert recursive types into unknown
        // types.
        this._seenTypeSymbols = new WeakSet();
        // Unlike convertSymbol, types aren't immediately interesting without being tied to something.
        // They don't emit events and thus can be converted synchronously.
        this.convertType = (typeOrNode) => {
            assert(this._checker, "convertType may only be called when a call to convert is in progress.");
            if (typeOrNode == null) {
                return new index_1.IntrinsicType("any");
            }
            if ("kind" in typeOrNode) {
                const converter = this._typeNodeConverters.get(typeOrNode.kind);
                if (converter) {
                    return converter.convert(this, typeOrNode);
                }
                this.logger.warn(`Missing type converter for kind ${typeOrNode.kind} (${ts.SyntaxKind[typeOrNode.kind]})`);
                return new index_1.UnknownType(typeOrNode.getText());
            }
            // IgnoreErrors is important, without it, we can't assert that we will get a node.
            const node = this._checker.typeToTypeNode(typeOrNode, void 0, ts.NodeBuilderFlags.IgnoreErrors);
            assert(node); // According to the TS source of typeToString, this is a bug if it does not hold.
            // HACK: This ought not be necessary, but we need some way to discover recursively
            // typed symbols which do not have type nodes. See the `recursive` symbol in the variables test.
            const symbol = typeOrNode.getSymbol();
            if (symbol) {
                if (node.kind !== ts.SyntaxKind.TypeReference &&
                    this._seenTypeSymbols.has(symbol)) {
                    const typeString = this._checker.typeToString(typeOrNode);
                    this.logger.verbose(`Refusing to recurse when converting type: ${typeString}`);
                    return new index_1.UnknownType(typeString);
                }
                this._seenTypeSymbols.add(symbol);
            }
            const converter = this._typeNodeConverters.get(node.kind);
            if (converter) {
                const result = converter.convertType(this, typeOrNode, node);
                if (symbol)
                    this._seenTypeSymbols.delete(symbol);
                return result;
            }
            this.logger.warn(`Missing type converter for type: ${this._checker.typeToString(typeOrNode)}`);
            return new index_1.UnknownType(this._checker.typeToString(typeOrNode));
        };
        nodes_1.addConverters(this);
        types_1.addTypeConverters(this);
    }
    get options() {
        return this.application.options;
    }
    get logger() {
        return this.application.logger;
    }
    get checker() {
        assert(this._checker, "type checker may only be accessed while conversion is in progress.");
        return this._checker;
    }
    get project() {
        assert(this._project, "project may only be accessed while conversion is in progress.");
        return this._project;
    }
    addReflectionConverter(converter) {
        for (const kind of converter.kind) {
            assert(!this._reflectionConverters.has(kind), `Duplicate converter for ${ts.SyntaxKind[kind]}`);
            this._reflectionConverters.set(kind, converter);
        }
    }
    addTypeNodeConverter(converter) {
        for (const kind of converter.kind) {
            assert(!this._reflectionConverters.has(kind), `Duplicate type node converter for ${ts.SyntaxKind[kind]}`);
            this._typeNodeConverters.set(kind, converter);
        }
    }
    async convert(program, entryPoints) {
        var _a, _b;
        const start = Date.now();
        const compilerOptions = program.getCompilerOptions();
        const rootDir = path_1.resolve((_b = (_a = compilerOptions.baseUrl) !== null && _a !== void 0 ? _a : compilerOptions.rootDir) !== null && _b !== void 0 ? _b : fs_1.getCommonDirectory(program.getRootFileNames()));
        const checker = (this._checker = program.getTypeChecker());
        const { name, readme } = await utils_1.discoverProjectInfo(rootDir, this.options.getValue("name"), this.options.getValue("readme"), this.options.getValue("includeVersion"));
        const project = (this._project = new index_1.ProjectReflection(name, readme));
        await this.emit(Converter.EVENT_BEGIN, project, program);
        const context = new context_1.Context(program, this, project, project);
        // Due to the order juggling necessary to only document each item once, the module
        // converter is not a standard converter. First, prepare the data for each entry point.
        const entries = array_1.filterMap(entryPoints, (entry) => {
            const file = program.getSourceFile(entry);
            if (!file) {
                this.logger.error(`Failed to find source file for entry point ${entry}`);
                return;
            }
            const moduleSymbol = this.getModuleSymbol(file, checker);
            if (!moduleSymbol) {
                this.logger.error(`Provided entry point ${entry} is not a module.`);
                return;
            }
            const name = path_1.relative(rootDir, file.fileName)
                .replace(/(\.d)?\.[tj]sx?$/, "")
                .replace(/\\/g, "/");
            // [resolved, exported]
            const exportSymbols = this.getExportsOfModule(file, checker).map((symbol) => symbol.flags & ts.SymbolFlags.Alias
                ? [checker.getAliasedSymbol(symbol), symbol]
                : [symbol, symbol]);
            return { name, file, moduleSymbol, exportSymbols };
        });
        // Now do a first pass of each module, documenting symbols which are directly exported.
        for (const { name, file, moduleSymbol, exportSymbols } of entries) {
            const entryStart = Date.now();
            this.logger.verbose(`First pass: ${name}`);
            const moduleReflection = entries.length === 1 ? project : new index_1.ModuleReflection(name);
            moduleReflection.comment = comments_1.getCommentForNodes([file]);
            project.registerReflection(moduleReflection, moduleSymbol);
            if (!moduleReflection.isProject()) {
                project.addChild(moduleReflection);
            }
            await this.emit(Converter.EVENT_MODULE_CREATED, moduleReflection, file);
            for (const exportSymbol of exportSymbols) {
                if (documentInFirstPass(exportSymbol[0], file)) {
                    await this.convertSymbol(exportSymbol[1], context.withContainer(moduleReflection));
                }
            }
            this.logger.verbose(`[Perf] First pass of ${name} took ${Date.now() - entryStart}ms`);
        }
        // Now do a second pass. This time, pick up symbols which were not directly exported and
        // can therefore be attached to any entry point.
        for (const { name, file, moduleSymbol, exportSymbols } of entries) {
            const entryStart = Date.now();
            this.logger.verbose(`Second pass: ${name}`);
            const moduleReflection = this._project
                .getReflectionsFromSymbol(moduleSymbol)
                .values()
                .next().value;
            for (const exportSymbol of exportSymbols) {
                if (!documentInFirstPass(exportSymbol[0], file)) {
                    await this.convertSymbol(exportSymbol[1], context.withContainer(moduleReflection));
                }
            }
            this.logger.verbose(`[Perf] Second pass of ${name} took ${Date.now() - entryStart}ms`);
        }
        await this.emit(Converter.EVENT_END, project);
        this._checker = undefined;
        this._project = undefined;
        this.logger.verbose(`[Perf] Conversion took ${Date.now() - start}ms`);
        return project;
    }
    async convertSymbol(symbol, context) {
        var _a, _b, _c, _d, _e;
        this.logger.verbose(`\tConverting symbol: ${symbol.name}`);
        const overrideName = symbol.flags & ts.SymbolFlags.Alias ? symbol.name : null;
        if (symbol.flags & ts.SymbolFlags.Alias) {
            symbol = context.checker.getAliasedSymbol(symbol);
        }
        const existingReflections = this.project.getReflectionsFromSymbol(symbol);
        if (existingReflections.size) {
            const target = existingReflections.values().next().value;
            const child = new index_1.ReferenceReflection(overrideName !== null && overrideName !== void 0 ? overrideName : symbol.name, target);
            this.project.registerReflection(child, symbol);
            // This can only happen with re-exports, for which it is safe.
            context.container.addChild(child);
            return;
        }
        const kinds = new Set((_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.map((d) => d.kind));
        if (kinds.has(ts.SyntaxKind.ClassDeclaration)) {
            // This is the one form of declaration merging we do, merging classes and interfaces.
            kinds.delete(ts.SyntaxKind.InterfaceDeclaration);
        }
        if (kinds.has(ts.SyntaxKind.GetAccessor)) {
            // Not exactly declaration merging... we treat accessors as one API item.
            kinds.delete(ts.SyntaxKind.SetAccessor);
        }
        for (const kind of kinds) {
            const converter = this._reflectionConverters.get(kind);
            if (!converter) {
                this.logger.verbose(`Missing converter for symbol ${symbol.name} with kind ${kind} (${ts.SyntaxKind[kind]})`);
                return;
            }
            const nodes = (_c = (_b = symbol.getDeclarations()) === null || _b === void 0 ? void 0 : _b.filter((d) => d.kind === kind)) !== null && _c !== void 0 ? _c : [];
            if (kind === ts.SyntaxKind.ClassDeclaration) {
                nodes.push(...((_e = (_d = symbol
                    .getDeclarations()) === null || _d === void 0 ? void 0 : _d.filter((d) => d.kind === ts.SyntaxKind.InterfaceDeclaration)) !== null && _e !== void 0 ? _e : []));
            }
            const child = await converter.convert(context, symbol, nodes);
            child.comment = comments_1.getCommentForNodes(nodes);
            child.name = overrideName !== null && overrideName !== void 0 ? overrideName : child.name;
            // TS isn't smart enough to know this is safe. The converters are carefully structured so that it is.
            context.container.addChild(child);
            await this.emit(Converter.EVENT_REFLECTION_CREATED, child, symbol, nodes);
        }
    }
    async convertTypeOrObject(typeOrNode) {
        // TODO: Fix this
        if (typeOrNode &&
            "kind" in typeOrNode &&
            ts.isTypeLiteralNode(typeOrNode)) {
            this.logger.verbose("TODO Fix convertTypeOrObject");
            return new object_1.ObjectReflection("TODO", [], [], []);
            // register
        }
        return this.convertType(typeOrNode);
    }
    /**
     * Wrapper around checker.getSymbolAtLocation which resolves to the declaration
     * symbol instead of any imported alias symbols.
     * @param node
     */
    getSymbolAtLocation(node) {
        const symbol = this.checker.getSymbolAtLocation(node);
        if (!symbol)
            return;
        return symbol.flags & ts.SymbolFlags.Alias
            ? this.checker.getAliasedSymbol(symbol)
            : symbol;
    }
    getModuleSymbol(file, checker) {
        let symbol = checker.getSymbolAtLocation(file);
        if (!symbol && file.flags & ts.NodeFlags.JavaScriptFile) {
            // TS INTERNAL: It seems like there really ought to be a way to do this without relying on the internal symbol property.
            // we need to do this for CommonJS users with module.exports=, exports.foo= https://stackoverflow.com/q/62865648/7186598
            symbol = file.symbol;
        }
        // This is a bit of a hack... but it lets us support some declaration files better.
        // Look for a global namespace defined in this file with the same name as the file name.
        if (!symbol) {
            const fileName = file.fileName.replace(/^.*[\\/](\w+)(?:\.d)?\.tsx?$/i, "$1");
            symbol = checker
                .getSymbolsInScope(file, ts.SymbolFlags.Module)
                .find((mod) => {
                var _a;
                return mod.name === fileName && ((_a = mod.getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((decl) => decl.getSourceFile() === file));
            });
        }
        return symbol;
    }
    getExportsOfModule(file, checker) {
        const symbol = this.getModuleSymbol(file, checker);
        return symbol ? checker.getExportsOfModule(symbol) : [];
    }
}
exports.Converter = Converter;
/** @event */
Converter.EVENT_BEGIN = "begin";
/** @event */
Converter.EVENT_END = "end";
/**
 * Fired for all reflection creation events except the creation of the main
 * project reflection and the module reflections contained within it.
 *
 * This cannot be fired for module reflections since, if a file is global,
 * there will not be a TypeScript symbol associated with it. Similarly, the
 * project reflection also has no single symbol associated with it. To listen
 * to the creation of a project reflection and module reflection, listen to
 * the {@link BEGIN} and {@link MODULE_CREATED} events, respectively.
 *
 * @event
 */
Converter.EVENT_REFLECTION_CREATED = "reflectionCreated";
/** @event */
Converter.EVENT_MODULE_CREATED = "moduleCreated";
function documentInFirstPass(symbol, file) {
    var _a;
    const declaration = (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a[0];
    if (!declaration) {
        return false;
    }
    // Only document items which are declared in this file in the first pass.
    return declaration.getSourceFile() === file;
}
//# sourceMappingURL=converter.js.map