"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTypeConverters = void 0;
const assert = require("assert");
const ts = require("typescript");
const M = require("../models");
const utils_1 = require("./utils");
const mapped_1 = require("../models/types/mapped");
function addTypeConverters(converter) {
    for (const typeNodeConverter of [
        arrayConverter,
        conditionalConverter,
        constructorConverter,
        exprWithTypeArgsConverter,
        functionConverter,
        indexedAccessConverter,
        inferConverter,
        intersectionConverter,
        keywordConverter,
        literalConverter,
        literalBooleanConverter,
        mappedConverter,
        namedTupleMemberConverter,
        operatorConverter,
        optionalConverter,
        parenthesizedConverter,
        predicateConverter,
        queryConverter,
        referenceConverter,
        thisConverter,
        tupleConverter,
        typeLiteralConverter,
        unionConverter,
    ]) {
        converter.addTypeNodeConverter(typeNodeConverter);
    }
}
exports.addTypeConverters = addTypeConverters;
// T[], Array<T>
const arrayConverter = {
    kind: [ts.SyntaxKind.ArrayType],
    convert(converter, node) {
        return new M.ArrayType(converter.convertType(node.elementType));
    },
    convertType(converter, type) {
        const params = converter.checker.getTypeArguments(type);
        assert(params.length === 1);
        return new M.ArrayType(converter.convertType(params[0]));
    },
};
// Check extends Extends ? True : False
const conditionalConverter = {
    kind: [ts.SyntaxKind.ConditionalType],
    convert(converter, node) {
        return new M.ConditionalType(converter.convertType(node.checkType), converter.convertType(node.extendsType), converter.convertType(node.trueType), converter.convertType(node.falseType));
    },
    convertType(converter, type) {
        return new M.ConditionalType(converter.convertType(type.checkType), converter.convertType(type.extendsType), converter.convertType(type.resolvedTrueType), converter.convertType(type.resolvedFalseType));
    },
};
// new () => Obj
const constructorConverter = {
    kind: [ts.SyntaxKind.ConstructorType],
    convert(converter, node) {
        var _a;
        return new M.ConstructorType(utils_1.convertTypeParameterDeclarations(converter, (_a = node.typeParameters) !== null && _a !== void 0 ? _a : []), utils_1.convertParameters(converter, node.parameters), converter.convertType(node.type));
    },
    convertType(converter, type) {
        var _a, _b;
        const signature = type.getConstructSignatures()[0];
        return new M.ConstructorType(utils_1.convertTypeParameters(converter, (_a = signature.getTypeParameters()) !== null && _a !== void 0 ? _a : []), utils_1.convertParameterSymbols(converter, (_b = signature.getParameters()) !== null && _b !== void 0 ? _b : []), converter.convertType(signature.getReturnType()));
    },
};
const exprWithTypeArgsConverter = {
    kind: [ts.SyntaxKind.ExpressionWithTypeArguments],
    convert(converter, node) {
        var _a, _b;
        const targetSymbol = converter.getSymbolAtLocation(node.expression);
        // Mixins... we might not have a symbol here.
        if (!targetSymbol) {
            return converter.convertType(converter.checker.getTypeAtLocation(node));
        }
        const parameters = (_b = (_a = node.typeArguments) === null || _a === void 0 ? void 0 : _a.map(converter.convertType)) !== null && _b !== void 0 ? _b : [];
        return new M.ReferenceType(targetSymbol.name, parameters, targetSymbol, false, converter.project);
    },
    // I don't think this is possible... expressions should always have type nodes.
    convertType: requestBugReport,
};
const functionConverter = {
    kind: [ts.SyntaxKind.FunctionType],
    convert(converter, node) {
        var _a;
        return new M.SignatureType(utils_1.convertTypeParameterDeclarations(converter, (_a = node.typeParameters) !== null && _a !== void 0 ? _a : []), utils_1.convertParameters(converter, node.parameters), converter.convertType(node.type));
    },
    convertType(converter, type) {
        var _a, _b;
        const signature = type.getCallSignatures()[0];
        return new M.SignatureType(utils_1.convertTypeParameters(converter, (_a = signature.getTypeParameters()) !== null && _a !== void 0 ? _a : []), utils_1.convertParameterSymbols(converter, (_b = signature.getParameters()) !== null && _b !== void 0 ? _b : []), converter.convertType(signature.getReturnType()));
    },
};
// T['a']
const indexedAccessConverter = {
    kind: [ts.SyntaxKind.IndexedAccessType],
    convert(converter, node) {
        return new M.IndexedAccessType(converter.convertType(node.objectType), converter.convertType(node.indexType));
    },
    convertType(converter, type) {
        // I don't think this is possible... these are never inferred.
        return requestBugReport(converter, type);
    },
};
// T extends infer Infer...
const inferConverter = {
    kind: [ts.SyntaxKind.InferType],
    convert(_converter, node) {
        return new M.InferredType(node.typeParameter.name.text);
    },
    convertType(_converter, type) {
        return new M.InferredType(type.symbol.name);
    },
};
// T & U
const intersectionConverter = {
    kind: [ts.SyntaxKind.IntersectionType],
    convert(converter, node) {
        return new M.IntersectionType(node.types.map(converter.convertType));
    },
    convertType(converter, type) {
        return new M.IntersectionType(type.types.map(converter.convertType));
    },
};
const keywordToTypeName = {
    [ts.SyntaxKind.AnyKeyword]: "any",
    [ts.SyntaxKind.UnknownKeyword]: "unknown",
    [ts.SyntaxKind.NumberKeyword]: "number",
    [ts.SyntaxKind.BigIntKeyword]: "bigint",
    [ts.SyntaxKind.ObjectKeyword]: "object",
    [ts.SyntaxKind.BooleanKeyword]: "boolean",
    [ts.SyntaxKind.StringKeyword]: "string",
    [ts.SyntaxKind.SymbolKeyword]: "symbol",
    [ts.SyntaxKind.VoidKeyword]: "void",
    [ts.SyntaxKind.UndefinedKeyword]: "undefined",
    [ts.SyntaxKind.NeverKeyword]: "never",
};
const keywordConverter = {
    kind: Object.keys(keywordToTypeName).map(Number),
    convert(_converter, node) {
        return new M.IntrinsicType(keywordToTypeName[node.kind]);
    },
    convertType(converter, type) {
        // Note: There is an internal `intrinsicName` property on `type` we could
        // use here, but `typeToString` isn't that much more expensive.
        return new M.IntrinsicType(converter.checker.typeToString(type));
    },
};
// Note: Literal types are not the same as type literals! SyntaxKind.TypeLiteral is an object.
// 1, 1e7, 100n, 'str'
// null (as of TS 4.0)
const literalConverter = {
    kind: [ts.SyntaxKind.LiteralType],
    convert(converter, node) {
        const literal = node.literal;
        switch (literal.kind) {
            case ts.SyntaxKind.TrueKeyword:
                return new M.LiteralType(true);
            case ts.SyntaxKind.FalseKeyword:
                return new M.LiteralType(false);
            case ts.SyntaxKind.NullKeyword:
                return new M.LiteralType(null);
            case ts.SyntaxKind.StringLiteral:
                return new M.LiteralType(literal.text);
            case ts.SyntaxKind.NumericLiteral:
                return new M.LiteralType(Number(literal.getText()));
            case ts.SyntaxKind.BigIntLiteral: {
                const negative = literal.text[0] === "-";
                const value = literal.text.replace(/^[+-]|n$/g, "");
                return new M.LiteralType({ negative, value });
            }
            case ts.SyntaxKind.PrefixUnaryExpression: {
                const operand = literal.operand;
                switch (operand.kind) {
                    case ts.SyntaxKind.NumericLiteral:
                        return new M.LiteralType(Number(literal.getText()));
                    case ts.SyntaxKind.BigIntLiteral:
                        // + cannot be present in a big int literal, this must be -123n
                        return new M.LiteralType({
                            value: operand.getText().replace("n", ""),
                            negative: true,
                        });
                }
            }
        }
        return requestBugReport(converter, literal);
    },
    convertType(_converter, type, node) {
        if (typeof type.value === "object") {
            return new M.LiteralType({
                value: type.value.base10Value,
                negative: type.value.negative,
            });
        }
        // This is unfortunate... `type.value` is undefined in this case.
        // Seems like a bug. https://github.com/microsoft/TypeScript/issues/26075
        switch (node.literal.kind) {
            case ts.SyntaxKind.TrueKeyword:
                return new M.LiteralType(true);
            case ts.SyntaxKind.FalseKeyword:
                return new M.LiteralType(false);
        }
        return new M.LiteralType(type.value);
    },
};
const literalBooleanConverter = {
    kind: [ts.SyntaxKind.TrueKeyword, ts.SyntaxKind.FalseKeyword],
    convert(_converter, node) {
        return new M.LiteralType(node.kind === ts.SyntaxKind.TrueKeyword);
    },
    convertType(_converter, _type, node) {
        return new M.LiteralType(node.kind === ts.SyntaxKind.TrueKeyword);
    },
};
const tokenToModifier = {
    [ts.SyntaxKind.ReadonlyKeyword]: mapped_1.OptionalModifier.Add,
    [ts.SyntaxKind.QuestionToken]: mapped_1.OptionalModifier.Add,
    [ts.SyntaxKind.PlusToken]: mapped_1.OptionalModifier.Add,
    [ts.SyntaxKind.MinusToken]: mapped_1.OptionalModifier.Remove,
};
const mappedConverter = {
    kind: [ts.SyntaxKind.MappedType],
    convert(converter, node) {
        const parameter = utils_1.convertTypeParameterDeclarations(converter, [
            node.typeParameter,
        ])[0];
        assert(parameter.constraint, "this should have caused a compiler error.");
        const optionalModifier = node.questionToken
            ? tokenToModifier[node.questionToken.kind]
            : mapped_1.OptionalModifier.None;
        const targetType = converter.convertType(node.type);
        return new M.MappedType(node.readonlyToken
            ? tokenToModifier[node.readonlyToken.kind]
            : mapped_1.OptionalModifier.None, parameter, optionalModifier, optionalModifier === mapped_1.OptionalModifier.Add
            ? utils_1.excludeUndefined(targetType)
            : targetType);
    },
    convertType(converter, type, node) {
        // This can happen if a generic function does not have a return type annotated.
        const parameter = utils_1.convertTypeParameters(converter, [type.typeParameter])[0];
        parameter.constraint = converter.convertType(type.constraintType);
        const optionalModifier = node.questionToken
            ? tokenToModifier[node.questionToken.kind]
            : mapped_1.OptionalModifier.None;
        const targetType = converter.convertType(node.type);
        return new M.MappedType(node.readonlyToken
            ? tokenToModifier[node.readonlyToken.kind]
            : mapped_1.OptionalModifier.None, parameter, optionalModifier, optionalModifier === mapped_1.OptionalModifier.Add
            ? utils_1.excludeUndefined(targetType)
            : targetType);
    },
};
const namedTupleMemberConverter = {
    kind: [ts.SyntaxKind.NamedTupleMember],
    convert(converter, node) {
        return new M.TupleNamedMemberType(node.name.text, hasQuestionToken(node), utils_1.excludeUndefined(converter.convertType(node.type)));
    },
    convertType: requestBugReport,
};
const operators = {
    [ts.SyntaxKind.ReadonlyKeyword]: "readonly",
    [ts.SyntaxKind.KeyOfKeyword]: "keyof",
    [ts.SyntaxKind.UniqueKeyword]: "unique",
};
// keyof T, readonly T, unique symbol
const operatorConverter = {
    kind: [ts.SyntaxKind.TypeOperator],
    convert(converter, node) {
        return new M.TypeOperatorType(converter.convertType(node.type), operators[node.operator]);
    },
    convertType(converter, type, node) {
        // readonly is only valid on array and tuple literal types.
        if (node.operator === ts.SyntaxKind.ReadonlyKeyword) {
            assert(isObjectType(type));
            const args = converter.checker
                .getTypeArguments(type)
                .map(converter.convertType);
            const inner = type.objectFlags & ts.ObjectFlags.Tuple
                ? new M.TupleType(args)
                : new M.ArrayType(args[0]);
            return new M.TypeOperatorType(inner, "readonly");
        }
        // keyof will only show up with generic functions, otherwise it gets eagerly
        // resolved to a union of strings.
        if (node.operator === ts.SyntaxKind.KeyOfKeyword) {
            // There's probably an interface for this somewhere... I couldn't find it.
            const targetType = type.type;
            return new M.TypeOperatorType(converter.convertType(targetType), "keyof");
        }
        // TS drops `unique` in `unique symbol` everywhere. If someone used it, we ought
        // to have a type node. This shouldn't ever happen.
        return requestBugReport(converter, type);
    },
};
// type Z = [string?]
//           ^^^^^^^
const optionalConverter = {
    kind: [ts.SyntaxKind.OptionalType],
    convert(converter, node) {
        return new M.OptionalType(converter.convertType(node.type));
    },
    // When we only have a type, this is never created. We can have optional types within
    // tuples on the type side, but they must be handled by the type converter for tuples.
    convertType: requestBugReport,
};
// Just collapse these...
// ((number))
const parenthesizedConverter = {
    kind: [ts.SyntaxKind.ParenthesizedType],
    convert(converter, node) {
        return converter.convertType(node.type);
    },
    convertType: requestBugReport,
};
// function foo(x: any): asserts x is String;
const predicateConverter = {
    kind: [ts.SyntaxKind.TypePredicate],
    convert(converter, node) {
        const assertsModifier = !!node.assertsModifier;
        const type = node.type ? converter.convertType(node.type) : undefined;
        return new M.PredicateType(node.parameterName.getText(), assertsModifier, type);
    },
    // These are not currently ever inferred. This might change in a future version of TS.
    convertType: requestBugReport,
};
// typeof Foo.bar
const queryConverter = {
    kind: [ts.SyntaxKind.TypeQuery],
    convert(converter, node) {
        const symbol = converter.getSymbolAtLocation(node.exprName);
        assert(symbol, `Query type failed to get a symbol for: ${node.exprName.getText()}. This is a bug.`);
        return new M.QueryType(new M.ReferenceType(symbol.name, [], symbol, true, converter.project));
    },
    convertType(converter, type) {
        const symbol = type.symbol;
        assert(symbol, `Query type failed to get a symbol for: ${converter.checker.typeToString(type)}. This is a bug.`);
        return new M.QueryType(new M.ReferenceType(symbol.name, [], symbol, true, converter.project));
    },
};
// Array<Foo>, Generic<A, B, C>
const referenceConverter = {
    kind: [ts.SyntaxKind.TypeReference],
    convert(converter, node) {
        var _a, _b;
        const targetSymbol = converter.getSymbolAtLocation(node.typeName);
        assert(targetSymbol, `Reference type failed to get a symbol for ${node.typeName.getText()}. This is probably a bug.`);
        const parameters = (_b = (_a = node.typeArguments) === null || _a === void 0 ? void 0 : _a.map((node) => converter.convertType(node))) !== null && _b !== void 0 ? _b : [];
        return new M.ReferenceType(targetSymbol.name, parameters, targetSymbol, false, converter.project);
    },
    convertType(converter, type) {
        var _a, _b;
        if (type.symbol) {
            return new M.ReferenceType(type.symbol.name, (_b = (_a = type.typeArguments) === null || _a === void 0 ? void 0 : _a.map(converter.convertType)) !== null && _b !== void 0 ? _b : [], type.symbol, false, converter.project);
        }
        return converter.convertType(type.target);
    },
};
// method(): this
const thisConverter = {
    kind: [ts.SyntaxKind.ThisType],
    convert() {
        return new M.IntrinsicType("this");
    },
    convertType() {
        return new M.IntrinsicType("this");
    },
};
// [T, U]
const tupleConverter = {
    kind: [ts.SyntaxKind.TupleType],
    convert(converter, node) {
        return new M.TupleType(node.elements.map(converter.convertType));
    },
    convertType(converter, type, node) {
        const elements = converter.checker.getTypeArguments(type);
        const tuple = new M.TupleType(elements.map(converter.convertType));
        // We might have optional elements, but we can't know this when converting
        // the elements... so deal with it here.
        for (let i = 0; i < elements.length; ++i) {
            if (ts.isOptionalTypeNode(node.elements[i])) {
                tuple.elements[i] = new M.OptionalType(utils_1.excludeUndefined(tuple.elements[i]));
            }
        }
        // When converting a tuple with named elements and we get the elements as types,
        // they lose their names, so we restore it afterwards. This is only a problem in
        // type land, and requires a somewhat contrived example to produce it, but it could happen.
        if (elements.length > 0 && ts.isNamedTupleMember(node.elements[0])) {
            for (let i = 0; i < elements.length; ++i) {
                const element = node.elements[i];
                tuple.elements[i] = new M.TupleNamedMemberType(element.name.text, hasQuestionToken(element), hasQuestionToken(element)
                    ? utils_1.excludeUndefined(tuple.elements[i])
                    : tuple.elements[i]);
            }
        }
        return tuple;
    },
};
// { a: string, (): string, new (): String }
const typeLiteralConverter = {
    kind: [ts.SyntaxKind.TypeLiteral],
    convert(converter, node) {
        const properties = node.members
            .filter(ts.isPropertySignature)
            .map((prop) => new M.PropertyType(getPropertyName(prop.name), utils_1.hasReadonlyModifier(prop), hasQuestionToken(prop), utils_1.excludeUndefined(converter.convertType(prop.type))));
        const signatures = node.members
            .filter(ts.isCallSignatureDeclaration)
            .map((node) => {
            var _a;
            return new M.SignatureType(utils_1.convertTypeParameterDeclarations(converter, (_a = node.typeParameters) !== null && _a !== void 0 ? _a : []), utils_1.convertParameters(converter, node.parameters), converter.convertType(node.type));
        });
        const constructSignatures = node.members
            .filter(ts.isConstructSignatureDeclaration)
            .map((node) => {
            var _a;
            return new M.ConstructorType(utils_1.convertTypeParameterDeclarations(converter, (_a = node.typeParameters) !== null && _a !== void 0 ? _a : []), utils_1.convertParameters(converter, node.parameters), converter.convertType(node.type));
        });
        if (properties.length === 0) {
            if (signatures.length === 1 && constructSignatures.length === 0) {
                return signatures[0];
            }
            if (signatures.length === 0 && constructSignatures.length === 1) {
                return constructSignatures[0];
            }
        }
        return new M.ObjectType(properties, signatures, constructSignatures);
    },
    convertType(converter, type) {
        const properties = type.getProperties().map((symbol) => {
            const type = converter.checker.getTypeOfSymbolAtLocation(symbol, symbol.valueDeclaration);
            return new M.PropertyType(symbol.name, utils_1.hasReadonlyModifier(symbol.valueDeclaration), Boolean(symbol.flags & ts.SymbolFlags.Optional), converter.convertType(type));
        });
        const signatures = type
            .getCallSignatures()
            .map((signature) => {
            var _a, _b;
            return new M.SignatureType(utils_1.convertTypeParameters(converter, (_a = signature.getTypeParameters()) !== null && _a !== void 0 ? _a : []), utils_1.convertParameterSymbols(converter, (_b = signature.getParameters()) !== null && _b !== void 0 ? _b : []), converter.convertType(signature.getReturnType()));
        });
        const constructSignatures = type
            .getConstructSignatures()
            .map((signature) => {
            var _a, _b;
            return new M.ConstructorType(utils_1.convertTypeParameters(converter, (_a = signature.getTypeParameters()) !== null && _a !== void 0 ? _a : []), utils_1.convertParameterSymbols(converter, (_b = signature.getParameters()) !== null && _b !== void 0 ? _b : []), converter.convertType(signature.getReturnType()));
        });
        return new M.ObjectType(properties, signatures, constructSignatures);
    },
};
// T | U
const unionConverter = {
    kind: [ts.SyntaxKind.UnionType],
    convert(converter, node) {
        return new M.UnionType(node.types.map(converter.convertType));
    },
    convertType(converter, type) {
        return new M.UnionType(type.types.map(converter.convertType));
    },
};
/// Helpers
function requestBugReport(converter, nodeOrType) {
    if ("kind" in nodeOrType) {
        converter.logger.warn(`Failed to convert type node with kind: ${ts.SyntaxKind[nodeOrType.kind]} and text ${nodeOrType.getText()}. Please report a bug.`);
        return new M.UnknownType(nodeOrType.getText());
    }
    else {
        const typeString = converter.checker.typeToString(nodeOrType);
        converter.logger.warn(`Failed to convert type: ${typeString}. Please report a bug.`);
        return new M.UnknownType(typeString);
    }
}
function isObjectType(type) {
    return typeof type.objectFlags === "number";
}
function hasQuestionToken(declaration) {
    return !!declaration.questionToken;
}
function getPropertyName(name) {
    switch (name.kind) {
        case ts.SyntaxKind.NumericLiteral:
        case ts.SyntaxKind.StringLiteral:
        case ts.SyntaxKind.Identifier:
            return name.text;
        case ts.SyntaxKind.ComputedPropertyName:
        case ts.SyntaxKind.PrivateIdentifier:
            // FUTURE: It would be nice to be able to represent these better.
            return name.getText();
    }
}
//# sourceMappingURL=types.js.map