"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.discoverProjectInfo = exports.getVisibility = exports.hasDotDotDotToken = exports.hasQuestionToken = exports.hasReadonlyModifier = exports.convertParameters = exports.convertTypeParameterDeclarations = exports.convertParameterSymbols = exports.convertTypeParameters = exports.excludeUndefined = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const ts = require("typescript");
const M = require("../models");
const models_1 = require("../models");
function excludeUndefined(type) {
    if (type.kind === models_1.TypeKind.Union) {
        const undefIndex = type.types.findIndex((t) => t.kind === models_1.TypeKind.Intrinsic && t.name === "undefined");
        if (undefIndex !== -1) {
            type.types.splice(undefIndex, 1);
        }
        if (type.types.length > 1) {
            return type;
        }
        else {
            return type.types[0];
        }
    }
    // If this isn't a union, nothing to exclude.
    return type;
}
exports.excludeUndefined = excludeUndefined;
function convertTypeParameters(converter, parameters) {
    return parameters.map((param) => {
        const constraintType = param.getConstraint();
        const constraint = constraintType
            ? converter.convertType(constraintType)
            : undefined;
        const defaultType = param.getDefault();
        const defaultValue = defaultType
            ? converter.convertType(defaultType)
            : undefined;
        return new M.TypeParameterType(param.symbol.name, constraint, defaultValue);
    });
}
exports.convertTypeParameters = convertTypeParameters;
function convertParameterSymbols(converter, parameters) {
    return parameters.map((symbol) => {
        const type = converter.convertType(converter.checker.getTypeOfSymbolAtLocation(symbol, symbol.valueDeclaration));
        return new M.SignatureParameterType(symbol.name, hasQuestionToken(symbol.valueDeclaration), hasDotDotDotToken(symbol.valueDeclaration), type);
    });
}
exports.convertParameterSymbols = convertParameterSymbols;
function convertTypeParameterDeclarations(converter, parameters) {
    return parameters.map((param) => {
        const constraint = param.constraint
            ? converter.convertType(param.constraint)
            : undefined;
        const defaultValue = param.default
            ? converter.convertType(param.default)
            : undefined;
        return new M.TypeParameterType(param.name.text, constraint, defaultValue);
    });
}
exports.convertTypeParameterDeclarations = convertTypeParameterDeclarations;
function convertParameters(converter, parameters) {
    return parameters.map((param, index) => {
        const name = ts.isIdentifier(param.name)
            ? param.name.text
            : `param${index}`;
        const type = converter.convertType(param.type);
        return new M.SignatureParameterType(name, hasQuestionToken(param), hasDotDotDotToken(param), type);
    });
}
exports.convertParameters = convertParameters;
function hasReadonlyModifier(declaration) {
    var _a, _b;
    return ((_b = (_a = declaration === null || declaration === void 0 ? void 0 : declaration.modifiers) === null || _a === void 0 ? void 0 : _a.some((mod) => mod.kind === ts.SyntaxKind.ReadonlyKeyword)) !== null && _b !== void 0 ? _b : false);
}
exports.hasReadonlyModifier = hasReadonlyModifier;
function hasQuestionToken(declaration) {
    return !!declaration.questionToken;
}
exports.hasQuestionToken = hasQuestionToken;
function hasDotDotDotToken(declaration) {
    return !!declaration.dotDotDotToken;
}
exports.hasDotDotDotToken = hasDotDotDotToken;
function getVisibility(declaration) {
    var _a;
    for (const { kind } of (_a = declaration.modifiers) !== null && _a !== void 0 ? _a : []) {
        if (kind === ts.SyntaxKind.PublicKeyword) {
            return "public" /* Public */;
        }
        else if (kind === ts.SyntaxKind.ProtectedKeyword) {
            return "protected" /* Protected */;
        }
        else if (kind === ts.SyntaxKind.PrivateKeyword) {
            return "private" /* Private */;
        }
    }
    return "public" /* Public */;
}
exports.getVisibility = getVisibility;
/**
 * Helper to find the project name and readme.
 */
async function discoverProjectInfo(rootDir, projectName, readmeFile, includeVersion) {
    // Find package.json to discover the project name.
    let currentDir = path_1.resolve(rootDir);
    let packageFile = "";
    const filesFound = () => !!packageFile || !!readmeFile;
    const reachedTopDirectory = () => currentDir === path_1.resolve(path_1.join(currentDir, ".."));
    while (!reachedTopDirectory() && !filesFound()) {
        const files = await fs_1.promises.readdir(currentDir);
        for (const file of files) {
            const lowerFile = file.toLowerCase();
            if (!readmeFile && lowerFile === "readme.md") {
                readmeFile = path_1.join(currentDir, file);
            }
            if (!packageFile && lowerFile === "package.json") {
                packageFile = path_1.join(currentDir, file);
            }
        }
        currentDir = path_1.resolve(path_1.join(currentDir, ".."));
    }
    let name = projectName;
    if (packageFile) {
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        const packageInfo = require(packageFile);
        if (!name) {
            name = String(packageInfo.name);
        }
        if (includeVersion) {
            name = `${name} - v${packageInfo.version}`;
        }
    }
    const readme = readmeFile && readmeFile !== "none"
        ? await fs_1.promises.readFile(readmeFile, "utf-8")
        : "";
    return { name, readme };
}
exports.discoverProjectInfo = discoverProjectInfo;
//# sourceMappingURL=utils.js.map