import type { SomeReflection, IndependentReflection, ModelToSerialized } from "./index";
import type { ProjectReflection } from "./project";
import type { Serializer, BaseSerialized } from "../../serialization";
import type { Comment } from "../comments";
/**
 * Reset the reflection id.
 *
 * Used by the test cases to ensure the reflection ids won't change between runs.
 * May also be used by external tools to reset IDs between calls to {@link Application.convert}
 * If you use this function, you must ensure that reflections created in one run are not mixed
 * with reflections created in another run.
 */
export declare function resetReflectionID(): void;
/**
 * Defines the available reflection kinds.
 */
export declare enum ReflectionKind {
    Project = 1,
    Module = 2,
    Namespace = 4,
    Enum = 8,
    EnumMember = 16,
    Variable = 32,
    Function = 64,
    Class = 128,
    Interface = 256,
    Object = 512,
    Property = 1024,
    Accessor = 2048,
    Method = 4096,
    Signature = 8192,
    Parameter = 16384,
    Alias = 32768,
    Reference = 65536
}
export declare namespace ReflectionKind {
    const All: ReflectionKind;
    const toKindArray: (kind: number) => number[];
    const toKindString: (kind: number) => string;
}
/**
 * Base class for all reflection classes.
 *
 * While generating a documentation, TypeDoc generates an instance of {@link ProjectReflection}
 * as the root for all reflections within the project.
 *
 * Note that several plugins extend this class, dynamically adding properties during conversion.
 * For example, the sources plugin adds the {@link sources} property. Plugins can use declaration
 * merging to add properties to the interface for typechecking.
 *
 * ### Reflection Creation
 * When a reflection is created and added to a project it must be registered with that reflection using
 * {@link ProjectReflection.registerReflection} so that references can be resolved. If the reflection is
 * not registered, then references will likely be broken.
 */
export declare abstract class Reflection {
    /**
     * Unique id of this reflection.
     */
    readonly id: number;
    /**
     * The parsed documentation comment attached to this reflection.
     * This will be set by the converter and mutated by the comment plugin during the reflectionCreated event.
     */
    comment?: Comment;
    /**
     * The symbol name of this reflection.
     */
    get name(): string;
    set name(value: string);
    /**
     * The original name of the TypeScript declaration.
     * In most cases, this will be the same as the reflection name. It will be
     * different for reflections describing a source file since TypeScript
     * includes the full absolute path as the name of a source file.
     */
    get originalName(): string;
    /**
     * The kind of this reflection.
     */
    abstract readonly kind: ReflectionKind;
    /**
     * Serialize this reflection to a JSON object.
     */
    abstract serialize(serializer: Serializer, init: BaseSerialized<SomeReflection>): ModelToSerialized<SomeReflection>;
    /**
     * The reflection this reflection is a child of. This is set by the container reflection
     * when a child is added to the reflection.
     */
    get parent(): SomeReflection | undefined;
    set parent(value: SomeReflection | undefined);
    /**
     * Gets the project associated with this reflection.
     * If a reflection has not been added to a project, this will return undefined.
     */
    get project(): ProjectReflection | undefined;
    private _name;
    private _originalName?;
    private _parent?;
    private _projectCache?;
    /**
     * Create a new Reflection instance.
     */
    constructor(name: string);
    /**
     * Test whether this reflection is of the given kind.
     */
    kindOf<K extends ReflectionKind>(...kinds: K[]): this is {
        kind: K;
    };
    /**
     * Return the full name of this reflection.
     *
     * The full name contains the name of this reflection and the names of all parent reflections.
     *
     * @param separator  Separator used to join the names of the reflections.
     * @returns The full name of this reflection.
     */
    getFullName(separator?: string): string;
    /**
     * Checks if the current reflection is the project.
     * This is used instead of `instanceof` to avoid circular dependencies.
     */
    isProject(): this is ProjectReflection;
    /**
     * Checks if the current reflection is some container reflection.
     * Provides a safer check than using `instanceof ContainerReflection` since the latter
     * will result in the `children` property being typed as `any[]`.
     */
    isContainer(): this is ContainerReflection<IndependentReflection>;
}
/**
 * Describes a reflection which may contain other reflections.
 *
 * Container reflections describe object-like constructs which contain properties, including modules,
 * namespaces, and classes. Notably, while a {@link SignatureReflection} may contain several reflections
 * describing its parameters, and a {@link FunctionReflection} may contain several signature reflections,
 * they are not container reflections.
 */
export declare abstract class ContainerReflection<Child extends IndependentReflection> extends Reflection {
    /**
     * The children of this reflection. The type of the children depends on what type the container is.
     * For example, or enums, children will be {@link EnumMemberReflection}, while namespaces can have
     * almost any member type.
     */
    children: Child[];
    /**
     * Adds the given child to this container and sets the parent link.
     * @param child
     */
    addChild(child: Child): void;
    /**
     * Removes the given child from this container, it may be added to
     * another container afterwards. Mutates child, removing the parent link.
     *
     * @param child
     */
    removeChild(child: Child): void;
    /**
     * Traverse member reflections contained within this reflection.
     *
     * This method will only yield reflections contained within the current reflection as
     * members, including namespace members, class members, and enum members. It *will not*
     * yield function parameter reflections since they aren't members of the signature.
     */
    [Symbol.iterator](): IterableIterator<Child>;
}
