"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContainerReflection = exports.Reflection = exports.ReflectionKind = exports.resetReflectionID = void 0;
const assert = require("assert");
const utils_1 = require("../../utils");
const enum_1 = require("../../utils/enum");
/**
 * Current reflection id. This provides a quick way to look up a reference to a reflection.
 * Reflections may have different IDs across different versions of TypeScript since they are
 * generated according to the order each reflection is created in. They might also be
 * nondeterministic if event handlers return promises which must be awaited.
 */
let REFLECTION_ID = 0;
/**
 * Reset the reflection id.
 *
 * Used by the test cases to ensure the reflection ids won't change between runs.
 * May also be used by external tools to reset IDs between calls to {@link Application.convert}
 * If you use this function, you must ensure that reflections created in one run are not mixed
 * with reflections created in another run.
 */
function resetReflectionID() {
    REFLECTION_ID = 0;
}
exports.resetReflectionID = resetReflectionID;
/**
 * Defines the available reflection kinds.
 */
var ReflectionKind;
(function (ReflectionKind) {
    // It isn't possible to use the bit shifting directly here.
    // The enum doesn't care, but it prevents dynamically using these members
    // as properties elsewhere. Members are assigned powers of two so that
    // Module or Namespace can be efficiently checked with `kind & (Module | Namespace)`
    ReflectionKind[ReflectionKind["Project"] = 1] = "Project";
    ReflectionKind[ReflectionKind["Module"] = 2] = "Module";
    ReflectionKind[ReflectionKind["Namespace"] = 4] = "Namespace";
    ReflectionKind[ReflectionKind["Enum"] = 8] = "Enum";
    ReflectionKind[ReflectionKind["EnumMember"] = 16] = "EnumMember";
    ReflectionKind[ReflectionKind["Variable"] = 32] = "Variable";
    ReflectionKind[ReflectionKind["Function"] = 64] = "Function";
    ReflectionKind[ReflectionKind["Class"] = 128] = "Class";
    ReflectionKind[ReflectionKind["Interface"] = 256] = "Interface";
    ReflectionKind[ReflectionKind["Object"] = 512] = "Object";
    ReflectionKind[ReflectionKind["Property"] = 1024] = "Property";
    ReflectionKind[ReflectionKind["Accessor"] = 2048] = "Accessor";
    ReflectionKind[ReflectionKind["Method"] = 4096] = "Method";
    ReflectionKind[ReflectionKind["Signature"] = 8192] = "Signature";
    ReflectionKind[ReflectionKind["Parameter"] = 16384] = "Parameter";
    ReflectionKind[ReflectionKind["Alias"] = 32768] = "Alias";
    ReflectionKind[ReflectionKind["Reference"] = 65536] = "Reference";
})(ReflectionKind = exports.ReflectionKind || (exports.ReflectionKind = {}));
(function (ReflectionKind) {
    const LAST_KIND = ReflectionKind.Reference;
    ReflectionKind.All = LAST_KIND * 2 - 1;
    ReflectionKind.toKindArray = enum_1.makeToKindArray(LAST_KIND);
    ReflectionKind.toKindString = enum_1.makeToKindString(ReflectionKind);
})(ReflectionKind = exports.ReflectionKind || (exports.ReflectionKind = {}));
/**
 * Base class for all reflection classes.
 *
 * While generating a documentation, TypeDoc generates an instance of {@link ProjectReflection}
 * as the root for all reflections within the project.
 *
 * Note that several plugins extend this class, dynamically adding properties during conversion.
 * For example, the sources plugin adds the {@link sources} property. Plugins can use declaration
 * merging to add properties to the interface for typechecking.
 *
 * ### Reflection Creation
 * When a reflection is created and added to a project it must be registered with that reflection using
 * {@link ProjectReflection.registerReflection} so that references can be resolved. If the reflection is
 * not registered, then references will likely be broken.
 */
class Reflection {
    /**
     * Create a new Reflection instance.
     */
    constructor(name) {
        this.id = REFLECTION_ID++;
        this._name = name;
    }
    /**
     * The symbol name of this reflection.
     */
    get name() {
        return this._name;
    }
    set name(value) {
        if (this._originalName === undefined) {
            this._originalName = this._name;
        }
        this._name = value;
    }
    /**
     * The original name of the TypeScript declaration.
     * In most cases, this will be the same as the reflection name. It will be
     * different for reflections describing a source file since TypeScript
     * includes the full absolute path as the name of a source file.
     */
    get originalName() {
        var _a;
        return (_a = this._originalName) !== null && _a !== void 0 ? _a : this._name;
    }
    /**
     * The reflection this reflection is a child of. This is set by the container reflection
     * when a child is added to the reflection.
     */
    get parent() {
        return this._parent;
    }
    set parent(value) {
        this._parent = value;
        this._projectCache = undefined;
    }
    /**
     * Gets the project associated with this reflection.
     * If a reflection has not been added to a project, this will return undefined.
     */
    get project() {
        var _a;
        if (this.isProject()) {
            return this;
        }
        if (this._projectCache !== undefined) {
            return this._projectCache;
        }
        return (this._projectCache = (_a = this.parent) === null || _a === void 0 ? void 0 : _a.project);
    }
    /**
     * Test whether this reflection is of the given kind.
     */
    kindOf(...kinds) {
        return kinds.some((kind) => (this.kind & kind) !== 0);
    }
    /**
     * Return the full name of this reflection.
     *
     * The full name contains the name of this reflection and the names of all parent reflections.
     *
     * @param separator  Separator used to join the names of the reflections.
     * @returns The full name of this reflection.
     */
    getFullName(separator = ".") {
        if (!this.parent || this.parent.isProject()) {
            return this.name;
        }
        else {
            return this.parent.getFullName(separator) + separator + this.name;
        }
    }
    /**
     * Checks if the current reflection is the project.
     * This is used instead of `instanceof` to avoid circular dependencies.
     */
    isProject() {
        return false;
    }
    /**
     * Checks if the current reflection is some container reflection.
     * Provides a safer check than using `instanceof ContainerReflection` since the latter
     * will result in the `children` property being typed as `any[]`.
     */
    isContainer() {
        return this instanceof ContainerReflection;
    }
}
exports.Reflection = Reflection;
/**
 * Describes a reflection which may contain other reflections.
 *
 * Container reflections describe object-like constructs which contain properties, including modules,
 * namespaces, and classes. Notably, while a {@link SignatureReflection} may contain several reflections
 * describing its parameters, and a {@link FunctionReflection} may contain several signature reflections,
 * they are not container reflections.
 */
class ContainerReflection extends Reflection {
    constructor() {
        super(...arguments);
        /**
         * The children of this reflection. The type of the children depends on what type the container is.
         * For example, or enums, children will be {@link EnumMemberReflection}, while namespaces can have
         * almost any member type.
         */
        this.children = [];
    }
    /**
     * Adds the given child to this container and sets the parent link.
     * @param child
     */
    addChild(child) {
        assert.strictEqual(child.parent, undefined, `${child.getFullName()} was added to two containers simultaneously.`);
        this.children.push(child);
        // TypeScript cannot verify this is safe because it cannot verify that SomeReflection
        // contains *every possible* subclass of ContainerReflection. However, practically, plugins
        // do not add new top level reflections and this can be verified manually.
        // I (GB) believe the unsafety here is worth the improved developer experience gained by
        // providing the type as a discriminated union.
        child.parent = this;
    }
    /**
     * Removes the given child from this container, it may be added to
     * another container afterwards. Mutates child, removing the parent link.
     *
     * @param child
     */
    removeChild(child) {
        assert.strictEqual(child.parent, this, `Tried to remove ${child.getFullName()} from ${this.getFullName()}, but child is not a child of this container.`);
        utils_1.removeIfPresent(this.children, child);
        child.parent = undefined;
    }
    /**
     * Traverse member reflections contained within this reflection.
     *
     * This method will only yield reflections contained within the current reflection as
     * members, including namespace members, class members, and enum members. It *will not*
     * yield function parameter reflections since they aren't members of the signature.
     */
    *[Symbol.iterator]() {
        yield* this.children;
    }
}
exports.ContainerReflection = ContainerReflection;
//# sourceMappingURL=abstract.js.map