import type * as ts from "typescript";
import type { IndependentReflection, TopLevelReflection } from ".";
import type { BaseSerialized, Serialized, Serializer } from "../../serialization";
import { ContainerReflection, Reflection, ReflectionKind } from "./abstract";
import type { ModuleReflection } from "./module";
/**
 * A reflection that represents the root of the project.
 *
 * This reflection will either contain {@link ModuleReflection}s for each entry point of the program
 * or, if the project only contains one module, that module will be elided and its children will be
 * placed directly within the project.
 */
export declare class ProjectReflection extends ContainerReflection<TopLevelReflection | ModuleReflection> {
    readonly kind = ReflectionKind.Project;
    private _symbolToReflection;
    private _referenceGraph?;
    /**
     * All independent reflections within the project.
     */
    private _reflections;
    /**
     * The contents of the `readme.md` file associated with this project.
     */
    readme: string;
    constructor(name: string, readme: string);
    /**
     * Registers the given reflection so that it can be quickly looked up by references.
     * Should be called for every independent reflection added to the project.
     */
    registerReflection(reflection: IndependentReflection, symbol: ts.Symbol): void;
    /**
     * Removes a reflection from the documentation. Can be used by plugins to filter reflections
     * out of the generated documentation. Has no effect if the reflection is not present in the
     * project.
     */
    removeReflection(reflection: Reflection): void;
    /**
     * Gets the reflection registered for the given reflection ID, or undefined if it is not present
     * in the project.
     */
    getReflectionById(id: number): IndependentReflection | undefined;
    /**
     * Gets the reflection registered for the given TypeScript symbol, or undefined if it is not present
     * in the project.
     */
    getReflectionsFromSymbol(symbol: ts.Symbol): Set<IndependentReflection>;
    serialize(_serializer: Serializer, init: BaseSerialized<ProjectReflection>): SerializedProjectReflection;
    /** @inheritdoc */
    isProject(): boolean;
    private getReferenceGraph;
}
export interface SerializedProjectReflection extends Serialized<ProjectReflection, never> {
}
