"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectReflection = void 0;
const abstract_1 = require("./abstract");
const reference_1 = require("./reference");
/**
 * A reflection that represents the root of the project.
 *
 * This reflection will either contain {@link ModuleReflection}s for each entry point of the program
 * or, if the project only contains one module, that module will be elided and its children will be
 * placed directly within the project.
 */
class ProjectReflection extends abstract_1.ContainerReflection {
    constructor(name, readme) {
        super(name);
        this.kind = abstract_1.ReflectionKind.Project;
        // Maps a TypeScript symbol to the reflection ID it refers to.
        // Note that one symbol might refer to multiple reflections in the case of references or merged declarations.
        this._symbolToReflection = new WeakMap();
        /**
         * All independent reflections within the project.
         */
        this._reflections = new Map();
        this.readme = readme;
    }
    /**
     * Registers the given reflection so that it can be quickly looked up by references.
     * Should be called for every independent reflection added to the project.
     */
    registerReflection(reflection, symbol) {
        var _a;
        this._referenceGraph = undefined;
        this._reflections.set(reflection.id, reflection);
        const set = (_a = this._symbolToReflection.get(symbol)) !== null && _a !== void 0 ? _a : new Set();
        set.add(reflection.id);
        this._symbolToReflection.set(symbol, set);
    }
    /**
     * Removes a reflection from the documentation. Can be used by plugins to filter reflections
     * out of the generated documentation. Has no effect if the reflection is not present in the
     * project.
     */
    removeReflection(reflection) {
        var _a, _b;
        // First cleanup references
        for (const id of (_a = this.getReferenceGraph().get(reflection.id)) !== null && _a !== void 0 ? _a : []) {
            const ref = this.getReflectionById(id);
            if (ref) {
                this.removeReflection(ref);
            }
        }
        this.getReferenceGraph().delete(reflection.id);
        // Remove children + references to children.
        if (reflection.isContainer()) {
            for (const child of reflection) {
                this.removeReflection(child);
            }
        }
        // Remove reflection from parent.
        if ((_b = reflection.parent) === null || _b === void 0 ? void 0 : _b.isContainer()) {
            // We know reflection is the right type because it is already a child.
            reflection.parent.removeChild(reflection);
        }
        this._reflections.delete(reflection.id);
    }
    /**
     * Gets the reflection registered for the given reflection ID, or undefined if it is not present
     * in the project.
     */
    getReflectionById(id) {
        return this._reflections.get(id);
    }
    /**
     * Gets the reflection registered for the given TypeScript symbol, or undefined if it is not present
     * in the project.
     */
    getReflectionsFromSymbol(symbol) {
        var _a;
        const reflections = new Set();
        for (const id of (_a = this._symbolToReflection.get(symbol)) !== null && _a !== void 0 ? _a : []) {
            const reflection = this.getReflectionById(id);
            if (reflection) {
                reflections.add(reflection);
            }
        }
        return reflections;
    }
    serialize(_serializer, init) {
        return init;
    }
    /** @inheritdoc */
    isProject() {
        return true;
    }
    getReferenceGraph() {
        var _a;
        if (!this._referenceGraph) {
            this._referenceGraph = new Map();
            for (const ref of this._reflections.values()) {
                if (ref instanceof reference_1.ReferenceReflection) {
                    const target = ref.resolve();
                    if (target) {
                        const refs = (_a = this._referenceGraph.get(target.id)) !== null && _a !== void 0 ? _a : [];
                        refs.push(ref.id);
                        this._referenceGraph.set(target.id, refs);
                    }
                }
            }
        }
        return this._referenceGraph;
    }
}
exports.ProjectReflection = ProjectReflection;
//# sourceMappingURL=project.js.map