import type { BaseSerialized, Serialized, Serializer } from "../../serialization";
import type { SomeType } from "../types";
import { Reflection, ReflectionKind } from "./abstract";
import type { ObjectReflection } from "./object";
import type { Visibility } from "./class";
/**
 * Describes a property of a {@link ClassReflection} or {@link InterfaceReflection}.
 *
 * The reflection flags may mark this property's visibility and modifiers.
 *
 * ```ts
 * interface Interface {
 *   a: number <-- Here
 * }
 * ```
 */
export declare class PropertyReflection extends Reflection {
    readonly kind = ReflectionKind.Property;
    /**
     * The type of this property.
     */
    type: SomeType | ObjectReflection;
    /**
     * The access level of this property.
     */
    visibility: Visibility;
    /**
     * True if this property is marked with the `readonly` modifier.
     */
    isReadonly: boolean;
    /**
     * If the property has an initializer, that initializer as a string.
     *
     * For the `bar` property of the following class, this would be set to `'bar'`.
     * ```ts
     * class Foo {
     *   bar = 'bar';
     * }
     * ```
     */
    defaultValue?: string;
    constructor(name: string, type: SomeType | ObjectReflection, visibility: Visibility, isReadonly: boolean, defaultValue?: string);
    serialize(serializer: Serializer, init: BaseSerialized<PropertyReflection>): SerializedPropertyReflection;
}
export interface SerializedPropertyReflection extends Serialized<PropertyReflection, "type" | "visibility" | "defaultValue" | "isReadonly"> {
}
/**
 * Describes a dynamic getter or setter property of a {@link ClassReflection}
 *
 * Note that it is possible for a dynamic property to have a setter without a getter.
 * This is likely a mistake and is treated as an error by API Extractor, but TypeDoc is
 * not a linter, this can be checked with ESLint's
 * {@link https://eslint.org/docs/rules/accessor-pairs | accessor-pairs} rule.
 *
 * Like API Extractor, TypeDoc treats getter/setter pairs as a single item in the
 * documentation. However, TypeDoc supports excluding members from the documentation
 * by including tags within the documentation comment. This means that doc comments on
 * the setter must be considered when creating this reflection. See the {@link CommentPlugin}
 * for the implementation details.
 *
 * ```ts
 * class Class {
 *   get foo(): number <-- Here
 * }
 * ```
 */
export declare class AccessorReflection extends Reflection {
    readonly kind = ReflectionKind.Accessor;
    /**
     * The type of this property.
     */
    type: SomeType | ObjectReflection;
    /**
     * The access level of this property.
     */
    visibility: Visibility;
    /**
     * True if this property has a getter.
     */
    hasGetter: boolean;
    /**
     * True if this property has a setter.
     */
    hasSetter: boolean;
    constructor(name: string, type: SomeType | ObjectReflection, visibility: Visibility, hasGetter: boolean, hasSetter: boolean);
    serialize(serializer: Serializer, init: BaseSerialized<AccessorReflection>): SerializedDynamicPropertyReflection;
}
export interface SerializedDynamicPropertyReflection extends Serialized<AccessorReflection, "type" | "visibility" | "hasGetter" | "hasSetter"> {
}
