"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessorReflection = exports.PropertyReflection = void 0;
const abstract_1 = require("./abstract");
/**
 * Describes a property of a {@link ClassReflection} or {@link InterfaceReflection}.
 *
 * The reflection flags may mark this property's visibility and modifiers.
 *
 * ```ts
 * interface Interface {
 *   a: number <-- Here
 * }
 * ```
 */
class PropertyReflection extends abstract_1.Reflection {
    constructor(name, type, visibility, isReadonly, defaultValue) {
        super(name);
        this.kind = abstract_1.ReflectionKind.Property;
        this.type = type;
        this.visibility = visibility;
        this.isReadonly = isReadonly;
        this.defaultValue = defaultValue;
    }
    serialize(serializer, init) {
        const result = {
            ...init,
            type: serializer.toObject(this.type),
            visibility: this.visibility,
            isReadonly: this.isReadonly,
        };
        if (typeof this.defaultValue === "string") {
            result.defaultValue = this.defaultValue;
        }
        return result;
    }
}
exports.PropertyReflection = PropertyReflection;
/**
 * Describes a dynamic getter or setter property of a {@link ClassReflection}
 *
 * Note that it is possible for a dynamic property to have a setter without a getter.
 * This is likely a mistake and is treated as an error by API Extractor, but TypeDoc is
 * not a linter, this can be checked with ESLint's
 * {@link https://eslint.org/docs/rules/accessor-pairs | accessor-pairs} rule.
 *
 * Like API Extractor, TypeDoc treats getter/setter pairs as a single item in the
 * documentation. However, TypeDoc supports excluding members from the documentation
 * by including tags within the documentation comment. This means that doc comments on
 * the setter must be considered when creating this reflection. See the {@link CommentPlugin}
 * for the implementation details.
 *
 * ```ts
 * class Class {
 *   get foo(): number <-- Here
 * }
 * ```
 */
class AccessorReflection extends abstract_1.Reflection {
    constructor(name, type, visibility, hasGetter, hasSetter) {
        super(name);
        this.kind = abstract_1.ReflectionKind.Accessor;
        this.type = type;
        this.visibility = visibility;
        this.hasGetter = hasGetter;
        this.hasSetter = hasSetter;
    }
    serialize(serializer, init) {
        return {
            ...init,
            visibility: this.visibility,
            hasGetter: this.hasGetter,
            hasSetter: this.hasSetter,
            type: serializer.toObject(this.type),
        };
    }
}
exports.AccessorReflection = AccessorReflection;
//# sourceMappingURL=property.js.map