"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReferenceReflection = void 0;
const assert = require("assert");
const abstract_1 = require("./abstract");
/**
 * Describes a reflection which does not exist at this location, but is referenced. Used for imported reflections.
 *
 * Like the compiler API, references are collapsed. This provides some performance benefits for large projects.
 *
 * ```ts
 * // a.ts
 * export const a = 1;
 * // b.ts
 * import { a } from './a';
 * // Here to avoid extra work we create a reference to the original reflection in module a
 * // instead of copying the reflection.
 * export { a };
 * ```
 */
class ReferenceReflection extends abstract_1.Reflection {
    /**
     * Creates a reference reflection.
     *
     * References can only be created for reflections which have already been registered.
     * If a reflection is removed from the project that this reflection is contained within,
     * the reference will fail to resolve.
     */
    constructor(name, reflection) {
        super(name);
        this.kind = abstract_1.ReflectionKind.Reference;
        this._target =
            reflection instanceof ReferenceReflection
                ? reflection._target
                : reflection.id;
    }
    /**
     * Tries to get the reflection that is referenced.
     */
    resolve() {
        assert(this.project, "Reference reflection has no project and is unable to resolve.");
        return this.project.getReflectionById(this._target);
    }
    serialize(_serializer, init) {
        var _a;
        return {
            ...init,
            target: (_a = this.resolve()) === null || _a === void 0 ? void 0 : _a.id,
        };
    }
}
exports.ReferenceReflection = ReferenceReflection;
//# sourceMappingURL=reference.js.map