import type { BaseSerialized, Serialized, Serializer } from "../../serialization";
import type { SomeType, TypeParameterType } from "../types/index";
import { Reflection, ReflectionKind, ContainerReflection } from "./abstract";
import type { ObjectReflection } from "./object";
import type { ParameterReflection } from "./parameter";
import type { Visibility } from "./class";
/**
 * Represents a function exported from a module or namespace.
 *
 * Functions attached to classes or interfaces are represented by a {@link MethodReflection}
 * Both arrow functions (`export const a = () => 1`) and function expressions
 * (`export const a = function(){}`) are converted to a function reflection.
 */
export declare class FunctionReflection extends ContainerReflection<SignatureReflection> {
    readonly kind = ReflectionKind.Function;
    serialize(_serializer: Serializer, base: BaseSerialized<FunctionReflection>): SerializedFunctionReflection;
}
export interface SerializedFunctionReflection extends Serialized<FunctionReflection, never> {
}
/**
 * Represents a method on a class or interface.
 */
export declare class MethodReflection extends ContainerReflection<SignatureReflection> {
    readonly kind = ReflectionKind.Method;
    visibility: Visibility;
    constructor(name: string, visibility: Visibility);
    /**
     * The method on the parent class or interface which this method overwrites.
     *
     * Note that this is appropriately named. Unless types are identical, subclasses may specify methods
     * which overwrite, rather than implement, parent interfaces. This can be partially mitigated with
     * strictFunctionTypes, but even then only applies to function properties, not class methods.
     *
     * ```ts
     * interface A { method(): string }
     * interface B extends A { method(): 'b' }
     * class Impl implements B {
     *   // Even though this implements A, which B extends, it does not meet the requirements for B.
     *   // So B's method() *overwrites* A's method().
     *   method() { // Error: Type 'string' is not assignable to type '"b"'
     *     return 'impl'
     *   }
     * }
     * ```
     *
     * This is a TypeScript design limitation. See Microsoft/TypeScript#22156. The default theme presents
     * methods as "overridden" instead of "overwritten", this is a compromise since most cases will not
     * intentionally overwrite methods. See GH#271.
     */
    get overwrites(): MethodReflection | undefined;
    set overwrites(value: MethodReflection | undefined);
    /**
     * The method on the parent class or interface which this signature is inherited from.
     */
    get inheritedFrom(): MethodReflection | undefined;
    set inheritedFrom(value: MethodReflection | undefined);
    /**
     * The method signature which this method is an implementation of.
     * Both {@link inheritedFrom} and {@link implementationOf} may be set, if the reflection
     * this reflection is inherited from is an implementation of some interface.
     */
    get implementationOf(): MethodReflection | undefined;
    set implementationOf(value: MethodReflection | undefined);
    private _overwrites?;
    private _inheritedFrom?;
    private _implementationOf?;
    serialize(_serializer: Serializer, init: BaseSerialized<MethodReflection>): SerializedMethodReflection;
}
export interface SerializedMethodReflection extends Serialized<MethodReflection, "visibility"> {
    overwrites?: number;
    inheritedFrom?: number;
    implementationOf?: number;
}
export declare class SignatureReflection extends Reflection {
    readonly kind = ReflectionKind.Signature;
    /**
     * The function parameters of this signature.
     */
    parameters: ParameterReflection[];
    /**
     * Any type parameters owned by this signature.
     */
    typeParameters: TypeParameterType[];
    /**
     * The return type of this signature.
     */
    returnType: SomeType | ObjectReflection;
    constructor(name: string, returnType: SomeType | ObjectReflection, parameters: ParameterReflection[], typeParameters: TypeParameterType[]);
    serialize(serializer: Serializer, init: BaseSerialized<SignatureReflection>): SerializedSignatureReflection;
}
export interface SerializedSignatureReflection extends Serialized<SignatureReflection, "parameters" | "typeParameters" | "returnType"> {
}
