"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignatureReflection = exports.MethodReflection = exports.FunctionReflection = void 0;
const abstract_1 = require("./abstract");
/**
 * Represents a function exported from a module or namespace.
 *
 * Functions attached to classes or interfaces are represented by a {@link MethodReflection}
 * Both arrow functions (`export const a = () => 1`) and function expressions
 * (`export const a = function(){}`) are converted to a function reflection.
 */
class FunctionReflection extends abstract_1.ContainerReflection {
    constructor() {
        super(...arguments);
        this.kind = abstract_1.ReflectionKind.Function;
    }
    serialize(_serializer, base) {
        return base;
    }
}
exports.FunctionReflection = FunctionReflection;
/**
 * Represents a method on a class or interface.
 */
class MethodReflection extends abstract_1.ContainerReflection {
    constructor(name, visibility) {
        super(name);
        this.kind = abstract_1.ReflectionKind.Method;
        this.visibility = visibility;
    }
    /**
     * The method on the parent class or interface which this method overwrites.
     *
     * Note that this is appropriately named. Unless types are identical, subclasses may specify methods
     * which overwrite, rather than implement, parent interfaces. This can be partially mitigated with
     * strictFunctionTypes, but even then only applies to function properties, not class methods.
     *
     * ```ts
     * interface A { method(): string }
     * interface B extends A { method(): 'b' }
     * class Impl implements B {
     *   // Even though this implements A, which B extends, it does not meet the requirements for B.
     *   // So B's method() *overwrites* A's method().
     *   method() { // Error: Type 'string' is not assignable to type '"b"'
     *     return 'impl'
     *   }
     * }
     * ```
     *
     * This is a TypeScript design limitation. See Microsoft/TypeScript#22156. The default theme presents
     * methods as "overridden" instead of "overwritten", this is a compromise since most cases will not
     * intentionally overwrite methods. See GH#271.
     */
    get overwrites() {
        var _a;
        return this._overwrites
            ? (_a = this.project) === null || _a === void 0 ? void 0 : _a.getReflectionById(this._overwrites)
            : undefined;
    }
    set overwrites(value) {
        this._overwrites = value === null || value === void 0 ? void 0 : value.id;
    }
    /**
     * The method on the parent class or interface which this signature is inherited from.
     */
    get inheritedFrom() {
        var _a;
        return this._inheritedFrom
            ? (_a = this.project) === null || _a === void 0 ? void 0 : _a.getReflectionById(this._inheritedFrom)
            : undefined;
    }
    set inheritedFrom(value) {
        this._inheritedFrom = value === null || value === void 0 ? void 0 : value.id;
    }
    /**
     * The method signature which this method is an implementation of.
     * Both {@link inheritedFrom} and {@link implementationOf} may be set, if the reflection
     * this reflection is inherited from is an implementation of some interface.
     */
    get implementationOf() {
        var _a;
        return this._implementationOf
            ? (_a = this.project) === null || _a === void 0 ? void 0 : _a.getReflectionById(this._implementationOf)
            : undefined;
    }
    set implementationOf(value) {
        this._inheritedFrom = value === null || value === void 0 ? void 0 : value.id;
    }
    serialize(_serializer, init) {
        const result = {
            ...init,
            visibility: this.visibility,
        };
        if (typeof this._overwrites === "number") {
            result.overwrites = this._overwrites;
        }
        if (typeof this._inheritedFrom === "number") {
            result.inheritedFrom = this._inheritedFrom;
        }
        if (typeof this._implementationOf === "number") {
            result.implementationOf = this._implementationOf;
        }
        return result;
    }
}
exports.MethodReflection = MethodReflection;
class SignatureReflection extends abstract_1.Reflection {
    constructor(name, returnType, parameters, typeParameters) {
        super(name);
        this.kind = abstract_1.ReflectionKind.Signature;
        this.returnType = returnType;
        this.parameters = parameters;
        this.typeParameters = typeParameters;
    }
    serialize(serializer, init) {
        return {
            ...init,
            parameters: serializer.toObjects(this.parameters),
            typeParameters: serializer.toObjects(this.typeParameters),
            returnType: serializer.toObject(this.returnType),
        };
    }
}
exports.SignatureReflection = SignatureReflection;
//# sourceMappingURL=signature.js.map