import type { Serializer, BaseSerialized, Serialized } from "../../serialization";
import { Type, TypeKind } from "./abstract";
import type { TypeParameterType } from "./type-parameter";
import type { SomeType } from ".";
export declare enum OptionalModifier {
    None = "none",
    Add = "add",
    Remove = "remove"
}
/**
 * Represents an intrinsic type like `string` or `boolean`.
 *
 * ```ts
 * let value: number;
 * type A = unknown;
 * ```
 */
export declare class MappedType extends Type {
    /** @inheritdoc */
    readonly kind = TypeKind.Mapped;
    /**
     * Represents the name of the mapped type.
     *
     * ```
     * { [K in keyof T]: string }
     *    ^^^^^^^^^^^^
     * ```
     *
     * Note that the constraint of the parameter in this case is `keyof T`
     */
    parameter: TypeParameterType & {
        constraint: SomeType;
    };
    /**
     * Represents the type that is being mapped to.
     *
     * ```
     * { [K in keyof T]: string }
     *                   ^^^^^^
     * ```
     */
    type: SomeType;
    /**
     * How optionality is modified by this mapped type.
     *
     * ```
     * { [K in keyof T]-?: string }
     *                 ^^
     * ```
     */
    optionalModifier: OptionalModifier;
    /**
     * How readonly properties are modified by this type.
     *
     * ```
     * { readonly [K in keyof T]: string }
     *   ^^^^^^^^
     * ```
     */
    readonlyModifier: OptionalModifier;
    constructor(readonlyModifier: OptionalModifier, parameter: TypeParameterType & {
        constraint: SomeType;
    }, optionalityModifier: OptionalModifier, type: SomeType);
    /** @inheritdoc */
    clone(): MappedType;
    /** @inheritdoc */
    stringify(): string;
    /** @inheritdoc */
    serialize(serializer: Serializer, init: BaseSerialized<MappedType>): SerializedMappedType;
}
export interface SerializedMappedType extends Serialized<MappedType, "readonlyModifier" | "parameter" | "optionalModifier" | "type"> {
}
