"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyType = exports.ObjectType = void 0;
const assert = require("assert");
const abstract_1 = require("./abstract");
const utils_1 = require("./utils");
/**
 * The most versatile type, describes any type of object with properties, methods,
 * signatures, and construct signatures.
 */
class ObjectType extends abstract_1.Type {
    constructor(
    /** Includes methods, like the TS Type. */
    properties, signatures, constructSignatures) {
        super();
        this.properties = properties;
        this.signatures = signatures;
        this.constructSignatures = constructSignatures;
        /** @inheritdoc */
        this.kind = abstract_1.TypeKind.Object;
    }
    /** @inheritdoc */
    clone() {
        return new ObjectType(utils_1.cloned(this.properties), utils_1.cloned(this.signatures), utils_1.cloned(this.constructSignatures));
    }
    /** @inheritdoc */
    stringify(wrapped) {
        // Try to display in a nice way.
        if (!this.properties.length &&
            !this.constructSignatures.length &&
            this.signatures.length === 1) {
            // Single signature, display as arrow: (params) => type
            return this.signatures[0].stringify(wrapped, true);
        }
        if (!this.properties.length &&
            !this.signatures.length &&
            this.constructSignatures.length === 1) {
            // Single construct signature, display as arrow: new (params) => type
            return this.constructSignatures[0].stringify(wrapped, true);
        }
        // Fall back to an object display, properties, then signatures, then construct signatures.
        const members = [
            ...this.properties.map((prop) => prop.stringify(false)),
            ...this.signatures.map((prop) => prop.stringify(false)),
            ...this.constructSignatures.map((prop) => prop.stringify(false)),
        ];
        return `{ ${members.join("; ")} }`;
    }
    /** @inheritdoc */
    serialize(serializer, init) {
        return {
            ...init,
            properties: serializer.toObjects(this.properties),
            signatures: serializer.toObjects(this.signatures),
            constructSignatures: serializer.toObjects(this.constructSignatures),
        };
    }
}
exports.ObjectType = ObjectType;
class PropertyType extends abstract_1.Type {
    constructor(name, isReadonly, isOptional, propertyType) {
        super();
        this.name = name;
        this.isReadonly = isReadonly;
        this.isOptional = isOptional;
        this.propertyType = propertyType;
        /** @inheritdoc */
        this.kind = abstract_1.TypeKind.Property;
    }
    /** @inheritdoc */
    clone() {
        return new PropertyType(this.name, this.isReadonly, this.isOptional, this.propertyType.clone());
    }
    /** @inheritdoc */
    stringify(wrapped) {
        assert(wrapped === false, "Property types are not considered wrapped when included in an object.");
        const frontModifier = this.isReadonly ? "readonly " : "";
        const tailModifier = this.isOptional ? "?" : "";
        return `${frontModifier}${this.name}${tailModifier}: ${this.propertyType}`;
    }
    /** @inheritdoc */
    serialize(serializer, init) {
        return {
            ...init,
            name: this.name,
            isReadonly: this.isReadonly,
            isOptional: this.isOptional,
            propertyType: serializer.toObject(this.propertyType),
        };
    }
}
exports.PropertyType = PropertyType;
//# sourceMappingURL=object.js.map