import type { BaseSerialized, Serialized, Serializer } from "../../serialization";
import { Type, TypeKind } from "./abstract";
import type { SomeType } from "./index";
import type { TypeParameterType } from "./type-parameter";
/**
 * Type which describes a signature.
 *
 * ```ts
 * type T = () => string
 * type U = <A>(arg: A) => A
 * ```
 */
export declare class SignatureType extends Type {
    typeParameters: TypeParameterType[];
    parameters: SignatureParameterType[];
    returnType: SomeType;
    /** @inheritdoc */
    readonly kind = TypeKind.Signature;
    constructor(typeParameters: TypeParameterType[], parameters: SignatureParameterType[], returnType: SomeType);
    /** @inheritdoc */
    clone(): SignatureType;
    /** @inheritdoc */
    stringify(wrapped: boolean, useArrow?: boolean): string;
    /** @inheritdoc */
    serialize(serializer: Serializer, init: BaseSerialized<SignatureType>): SerializedSignatureType;
}
export interface SerializedSignatureType extends Serialized<SignatureType, "typeParameters" | "parameters" | "returnType"> {
}
/**
 * Type which describes a construct signature.
 *
 * ```ts
 * type T = new () => string
 * type U = new <A extends new () => String>(arg: A) => String
 * ```
 */
export declare class ConstructorType extends Type {
    typeParameters: TypeParameterType[];
    parameters: SignatureParameterType[];
    returnType: SomeType;
    /** @inheritdoc */
    readonly kind = TypeKind.Constructor;
    constructor(typeParameters: TypeParameterType[], parameters: SignatureParameterType[], returnType: SomeType);
    /** @inheritdoc */
    clone(): ConstructorType;
    /** @inheritdoc */
    stringify(wrapped: boolean, useArrow?: boolean): string;
    /** @inheritdoc */
    serialize(serializer: Serializer, init: BaseSerialized<ConstructorType>): SerializedConstructorType;
}
export interface SerializedConstructorType extends Serialized<ConstructorType, "typeParameters" | "parameters" | "returnType"> {
}
/**
 * Type which describes a parameter of a signature.
 */
export declare class SignatureParameterType extends Type {
    name: string;
    isOptional: boolean;
    isRest: boolean;
    parameterType: SomeType;
    /** @inheritdoc */
    readonly kind = TypeKind.SignatureParameter;
    constructor(name: string, isOptional: boolean, isRest: boolean, parameterType: SomeType);
    /** @inheritdoc */
    clone(): SignatureParameterType;
    /** @inheritdoc */
    stringify(wrapped: boolean): string;
    /** @inheritdoc */
    serialize(serializer: Serializer, init: BaseSerialized<SignatureParameterType>): SerializedSignatureParameterType;
}
export interface SerializedSignatureParameterType extends Serialized<SignatureParameterType, "name" | "isOptional" | "isRest" | "parameterType"> {
}
