"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SignatureParameterType = exports.ConstructorType = exports.SignatureType = void 0;
const assert = require("assert");
const abstract_1 = require("./abstract");
const utils_1 = require("./utils");
/**
 * Type which describes a signature.
 *
 * ```ts
 * type T = () => string
 * type U = <A>(arg: A) => A
 * ```
 */
class SignatureType extends abstract_1.Type {
    constructor(typeParameters, parameters, returnType) {
        super();
        this.typeParameters = typeParameters;
        this.parameters = parameters;
        this.returnType = returnType;
        /** @inheritdoc */
        this.kind = abstract_1.TypeKind.Signature;
    }
    /** @inheritdoc */
    clone() {
        return new SignatureType(utils_1.cloned(this.typeParameters), utils_1.cloned(this.parameters), this.returnType.clone());
    }
    /** @inheritdoc */
    stringify(wrapped, useArrow = false) {
        const typeParameters = this.typeParameters.map(String).join(", ");
        const parameters = this.parameters.map(String).join(", ");
        const returnIndicator = useArrow ? ": " : " => ";
        return utils_1.wrap(wrapped, (typeParameters ? `<${typeParameters}>` : "") +
            `(${parameters})${returnIndicator}${this.returnType}`);
    }
    /** @inheritdoc */
    serialize(serializer, init) {
        return {
            ...init,
            typeParameters: serializer.toObjects(this.typeParameters),
            parameters: serializer.toObjects(this.parameters),
            returnType: serializer.toObject(this.returnType),
        };
    }
}
exports.SignatureType = SignatureType;
/**
 * Type which describes a construct signature.
 *
 * ```ts
 * type T = new () => string
 * type U = new <A extends new () => String>(arg: A) => String
 * ```
 */
class ConstructorType extends abstract_1.Type {
    constructor(typeParameters, parameters, returnType) {
        super();
        this.typeParameters = typeParameters;
        this.parameters = parameters;
        this.returnType = returnType;
        /** @inheritdoc */
        this.kind = abstract_1.TypeKind.Constructor;
    }
    /** @inheritdoc */
    clone() {
        return new ConstructorType(utils_1.cloned(this.typeParameters), utils_1.cloned(this.parameters), this.returnType.clone());
    }
    /** @inheritdoc */
    stringify(wrapped, useArrow = false) {
        const typeParameters = this.typeParameters.map(String).join(", ");
        const parameters = this.parameters.map(String).join(", ");
        const returnIndicator = useArrow ? ": " : " => ";
        return utils_1.wrap(wrapped, "new " +
            (typeParameters ? `<${typeParameters}>` : "") +
            `(${parameters})${returnIndicator}${this.returnType}`);
    }
    /** @inheritdoc */
    serialize(serializer, init) {
        return {
            ...init,
            typeParameters: serializer.toObjects(this.typeParameters),
            parameters: serializer.toObjects(this.parameters),
            returnType: serializer.toObject(this.returnType),
        };
    }
}
exports.ConstructorType = ConstructorType;
/**
 * Type which describes a parameter of a signature.
 */
class SignatureParameterType extends abstract_1.Type {
    constructor(name, isOptional, isRest, parameterType) {
        super();
        this.name = name;
        this.isOptional = isOptional;
        this.isRest = isRest;
        this.parameterType = parameterType;
        /** @inheritdoc */
        this.kind = abstract_1.TypeKind.SignatureParameter;
    }
    /** @inheritdoc */
    clone() {
        return new SignatureParameterType(this.name, this.isOptional, this.isRest, this.parameterType.clone());
    }
    /** @inheritdoc */
    stringify(wrapped) {
        assert(wrapped === false, "SignatureParameterTypes may not be contained within other types.");
        return ((this.isRest ? "..." : "") +
            this.name +
            (this.isOptional ? "?" : "") +
            ": " +
            this.parameterType);
    }
    /** @inheritdoc */
    serialize(serializer, init) {
        return {
            ...init,
            name: this.name,
            isOptional: this.isOptional,
            isRest: this.isRest,
            parameterType: serializer.toObject(this.parameterType),
        };
    }
}
exports.SignatureParameterType = SignatureParameterType;
//# sourceMappingURL=signature.js.map