"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HostLinkResolver = exports.BitbucketHost = exports.GitHubHost = exports.Host = void 0;
const assert = require("assert");
const shell = require("shelljs");
const path_1 = require("path");
class Host {
    constructor(files) {
        this._files = new Set(files);
    }
    /**
     * Check if this host builder supports the given remote.
     * @param remote
     */
    static supports(remote) {
        return this.HOST_REGEX.test(remote);
    }
    /**
     * Check if the given file is within this host.
     * @param file the absolute path to the file
     */
    containsFile(file) {
        return this._files.has(file);
    }
}
exports.Host = Host;
Host.HOST_REGEX = /^/;
/**
 * Create links to files in GitHub repositories.
 *
 * Handles remotes in the format:
 * - https://github.com/project/repository.git
 * - git@github.company.com:project/repository.git
 */
class GitHubHost extends Host {
    constructor(repoRoot, remote, files, gitRevision) {
        super(files);
        this.repoRoot = repoRoot;
        this.gitRevision = gitRevision;
        const match = remote.match(GitHubHost.HOST_REGEX);
        assert(match);
        this.hostname = match[1];
        this.project = match[2];
        this.repository = match[3];
    }
    createLink(file, line) {
        return [
            `https://${this.hostname}`,
            this.project,
            this.repository,
            "blob",
            this.gitRevision,
            `${path_1.posix.relative(this.repoRoot, file)}#L${line}`,
        ].join("/");
    }
}
exports.GitHubHost = GitHubHost;
GitHubHost.HOST_REGEX = /(github(?:\.[a-z]+)*\.com)[:/]([\w-]+)\/([\w-]+)/;
/**
 * Create links to files in Bitbucket repositories.
 *
 * Handles remotes in the format:
 * - git@bitbucket.org:project/repository.git
 * - https://username@bitbucket.org/project/repository.git
 */
class BitbucketHost extends Host {
    constructor(repoRoot, remote, files, gitRevision) {
        super(files);
        this.repoRoot = repoRoot;
        this.gitRevision = gitRevision;
        const match = remote.match(BitbucketHost.HOST_REGEX);
        assert(match);
        this.project = match[1];
        this.repository = match[2];
    }
    createLink(file, line) {
        return [
            "https://bitbucket.org",
            this.project,
            this.repository,
            "src",
            this.gitRevision,
            `${path_1.posix.relative(this.repoRoot, file)}#lines-${line}`,
        ].join("/");
    }
}
exports.BitbucketHost = BitbucketHost;
BitbucketHost.HOST_REGEX = /@bitbucket.org[/:]([\w-]+)\/([\w-]+)/;
const HOST_BUILDERS = [GitHubHost, BitbucketHost];
class HostLinkResolver {
    constructor(gitRemote, gitRevision) {
        this.gitRemote = gitRemote;
        this.gitRevision = gitRevision;
        this.hosts = [];
        // Files known not to have a link
        this.ignored = new Set();
    }
    tryGetUrl(file, line) {
        if (this.ignored.has(file)) {
            return;
        }
        let host = this.hosts.find((host) => host.containsFile(file));
        if (host) {
            return host.createLink(file, line);
        }
        host = this._tryCreateHost(file);
        if (host) {
            this.hosts.push(host);
            return host.createLink(file, line);
        }
        this.ignored.add(file);
    }
    _tryCreateHost(file) {
        shell.pushd(path_1.dirname(file));
        const repoRoot = shell.exec(`git rev-parse --show-toplevel`);
        shell.popd();
        if (repoRoot.code !== 0) {
            return;
        } // Not a git repository
        shell.pushd(repoRoot);
        const remote = shell.exec(`git remote get-url ${this.gitRemote}`);
        if (remote.code !== 0) {
            shell.popd();
            return; // Remote does not exist.
        }
        // Shouldn't ever fail.
        const files = shell
            .exec("git ls-files")
            .split("\n")
            .map((file) => path_1.join(repoRoot.trim(), file).replace(/\\/g, "/"));
        shell.popd();
        for (const builder of HOST_BUILDERS) {
            if (builder.supports(remote)) {
                return new builder(repoRoot.trim(), remote.trim(), files, this.gitRevision);
            }
        }
    }
}
exports.HostLinkResolver = HostLinkResolver;
//# sourceMappingURL=hosts.js.map