"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultTemplates = void 0;
const assert = require("assert");
const preact_1 = require("preact");
const models_1 = require("../models");
const type_templates_1 = require("./type-templates");
// See the Templates interface for doc comments.
// TODO: Lots to do to make this good yet.
const kindNames = {
    [models_1.ReflectionKind.Project]: "",
    [models_1.ReflectionKind.Module]: "Module",
    [models_1.ReflectionKind.Namespace]: "Namespace",
    [models_1.ReflectionKind.Enum]: "Enumeration",
    [models_1.ReflectionKind.EnumMember]: "Enumeration Member",
    [models_1.ReflectionKind.Variable]: "Variable",
    [models_1.ReflectionKind.Function]: "Function",
    [models_1.ReflectionKind.Class]: "Class",
    [models_1.ReflectionKind.Interface]: "Interface",
    [models_1.ReflectionKind.Object]: "Object",
    [models_1.ReflectionKind.Property]: "Property",
    [models_1.ReflectionKind.Accessor]: "Accessor",
    [models_1.ReflectionKind.Method]: "Method",
    [models_1.ReflectionKind.Signature]: "Signature",
    [models_1.ReflectionKind.Parameter]: "Parameter",
    [models_1.ReflectionKind.Alias]: "Type Alias",
    [models_1.ReflectionKind.Reference]: "Reference",
};
exports.DefaultTemplates = {
    Page(props) {
        const { reflection, templates, hooks } = props;
        assert(reflection.project);
        return (preact_1.createElement("html", null,
            preact_1.createElement("head", null,
                hooks.emit("head.begin", reflection),
                preact_1.createElement("meta", { charSet: "utf-8" }),
                preact_1.createElement("meta", { name: "description", content: `Documentation for ${reflection.project.name}` }),
                preact_1.createElement("meta", { name: "viewport", content: "width=device-width, initial-scale=1" }),
                preact_1.createElement("meta", { name: "generator", content: "typedoc" }),
                preact_1.createElement("title", null, reflection.isProject()
                    ? reflection.name
                    : `${reflection.name} | ${reflection.project.name}`),
                hooks.emit("head.end", reflection)),
            preact_1.createElement("body", null,
                hooks.emit("body.begin", reflection),
                preact_1.createElement(templates.Toolbar, Object.assign({}, props)),
                preact_1.createElement("main", null,
                    preact_1.createElement(templates.Header, Object.assign({}, props)),
                    preact_1.createElement(templates.Navigation, Object.assign({}, props)),
                    preact_1.createElement("section", { id: "main" },
                        preact_1.createElement(templates.Reflection, Object.assign({}, props))),
                    preact_1.createElement(templates.Footer, Object.assign({}, props))),
                hooks.emit("body.end", reflection))));
    },
    Toolbar(props) {
        const { reflection, router } = props;
        assert(reflection.project);
        return (preact_1.createElement("div", { class: "toolbar" },
            preact_1.createElement("a", { href: router.createLink(reflection, reflection.project) }, reflection.project.name),
            preact_1.createElement("input", { id: "search", placeholder: "Click or press S for search" }),
            preact_1.createElement("select", { id: "theme" },
                preact_1.createElement("option", { value: "native" }, "Native"),
                preact_1.createElement("option", { value: "dark" }, "Dark"),
                preact_1.createElement("option", { value: "light" }, "Light"))));
    },
    Header(props) {
        const { templates, reflection } = props;
        let displayName = reflection.name;
        if (reflection.kindOf(models_1.ReflectionKind.Class, models_1.ReflectionKind.Interface) &&
            reflection.typeParameters.length) {
            displayName += `<${reflection.typeParameters
                .map((param) => param.toString())
                .join(", ")}>`;
        }
        return (preact_1.createElement("header", null,
            preact_1.createElement("div", { class: "title" },
                preact_1.createElement(templates.Breadcrumbs, Object.assign({}, props)),
                preact_1.createElement("h1", null,
                    kindNames[reflection.kind],
                    " ",
                    displayName))));
    },
    Navigation({ reflection, router }) {
        let navMembers;
        if (router.hasOwnDocument(reflection) && reflection.isContainer()) {
            navMembers = reflection.children;
        }
        else {
            assert(reflection.parent && reflection.parent.isContainer());
            navMembers = reflection.parent.children;
        }
        return (preact_1.createElement("nav", null, navMembers.map((member) => (preact_1.createElement("a", { href: router.createLink(reflection, member) }, member.name)))));
    },
    Breadcrumbs({ reflection, router }) {
        const path = [];
        let iter = reflection.parent;
        while (iter) {
            path.unshift(iter);
            iter = iter.parent;
        }
        if (path.length < 1) {
            return preact_1.createElement(preact_1.Fragment, null);
        }
        return (preact_1.createElement("ul", { class: "breadcrumbs" }, path.map((refl) => (preact_1.createElement("li", null,
            preact_1.createElement("a", { href: router.createLink(reflection, refl) }, refl.name))))));
    },
    Footer() {
        // TODO hideGenerator, legend
        return (preact_1.createElement("footer", null,
            preact_1.createElement("p", null,
                "Generated using",
                " ",
                preact_1.createElement("a", { href: "https://typedoc.org/", target: "_blank" }, "TypeDoc"))));
    },
    // TODO: This needs to group according to categories.
    PageChildren(props) {
        const { router, reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null, router.getChildrenInPage(reflection).map((child) => (preact_1.createElement(templates.Reflection, Object.assign({}, props, { reflection: child, displayName: `${kindNames[child.kind]} ${child.name}` }))))));
    },
    Reflection(props) {
        const { reflection, templates, hooks, router, highlighter, parseMarkdown, displayName, } = props;
        const templateMap = {
            [models_1.ReflectionKind.Project]: templates.Project,
            [models_1.ReflectionKind.Module]: templates.Module,
            [models_1.ReflectionKind.Namespace]: templates.Namespace,
            [models_1.ReflectionKind.Enum]: templates.Enum,
            [models_1.ReflectionKind.EnumMember]: templates.EnumMember,
            [models_1.ReflectionKind.Variable]: templates.Variable,
            [models_1.ReflectionKind.Function]: templates.Function,
            [models_1.ReflectionKind.Class]: templates.Class,
            [models_1.ReflectionKind.Interface]: templates.Interface,
            [models_1.ReflectionKind.Object]: templates.Object,
            [models_1.ReflectionKind.Property]: templates.Property,
            [models_1.ReflectionKind.Accessor]: templates.Accessor,
            [models_1.ReflectionKind.Method]: templates.Method,
            [models_1.ReflectionKind.Signature]: templates.Signature,
            [models_1.ReflectionKind.Parameter]: templates.Parameter,
            [models_1.ReflectionKind.Alias]: templates.TypeAlias,
            [models_1.ReflectionKind.Reference]: templates.Reference,
        };
        const Template = templateMap[reflection.kind];
        if (!Template) {
            const kindString = reflection.kind in models_1.ReflectionKind
                ? models_1.ReflectionKind.toKindString(reflection.kind)
                : `${reflection.kind}`;
            throw new Error(`Invalid reflection kind ${kindString} for reflection ${reflection.getFullName()}`);
        }
        const slug = router.createSlug(reflection) || "#";
        return (preact_1.createElement(preact_1.Fragment, null,
            hooks.emit("reflection.before", reflection),
            preact_1.createElement("div", { class: `reflection reflection-${models_1.ReflectionKind.toKindString(reflection.kind)}`, id: slug },
                displayName && (preact_1.createElement("h3", null,
                    preact_1.createElement("a", { href: "#" + slug }, "#"),
                    " ",
                    displayName)),
                hooks.emit("reflection.begin", reflection),
                preact_1.createElement(Template
                // Don't just use {...props} here because we don't want to pass displayName through
                , { 
                    // Don't just use {...props} here because we don't want to pass displayName through
                    templates: templates, hooks: hooks, router: router, parseMarkdown: parseMarkdown, highlighter: highlighter, 
                    // Discriminated unions don't play nicely here... TS infers the required type to be `never`
                    reflection: reflection }),
                hooks.emit("reflection.end", reflection)),
            hooks.emit("reflection.after", reflection)));
    },
    Comment({ reflection, parseMarkdown }) {
        var _a;
        const comment = reflection.comment;
        if (!comment) {
            return preact_1.createElement(preact_1.Fragment, null);
        }
        const parsed = parseMarkdown(`${comment.shortText}\n\n${comment.text}`, reflection);
        const rendered = (preact_1.createElement("div", { class: "comment markdown", dangerouslySetInnerHTML: { __html: parsed } }));
        if ((_a = comment.tags) === null || _a === void 0 ? void 0 : _a.length) {
            return (preact_1.createElement(preact_1.Fragment, null,
                rendered,
                preact_1.createElement("dl", { class: "comment-tags" }, comment.tags.map((tag) => (preact_1.createElement(preact_1.Fragment, null,
                    preact_1.createElement("dt", null, tag.tagName + (tag.paramName ? ` ${tag.paramName}` : "")),
                    preact_1.createElement("dd", { dangerouslySetInnerHTML: {
                            __html: parseMarkdown(tag.text, reflection),
                        } })))))));
        }
        return rendered;
    },
    Project(props) {
        const { reflection, templates, parseMarkdown } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement("div", { class: "markdown", dangerouslySetInnerHTML: {
                    __html: parseMarkdown(reflection.readme, reflection),
                } }),
            preact_1.createElement(templates.PageChildren, Object.assign({}, props))));
    },
    Module(props) {
        const { templates } = props;
        // TODO: Index.
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement(templates.Comment, Object.assign({}, props)),
            preact_1.createElement(templates.PageChildren, Object.assign({}, props))));
    },
    Namespace(props) {
        const { templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement(templates.Comment, Object.assign({}, props)),
            preact_1.createElement(templates.PageChildren, Object.assign({}, props))));
    },
    Interface(props) {
        const { templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement(templates.Comment, Object.assign({}, props)),
            preact_1.createElement(templates.PageChildren, Object.assign({}, props))));
    },
    TypeAlias(props) {
        const { reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            "type ",
            reflection.name,
            preact_1.createElement(templates.TypeParameters, Object.assign({}, props, { params: reflection.typeParameters })),
            " = ",
            preact_1.createElement(templates.Type, Object.assign({}, props, { type: reflection.type })),
            preact_1.createElement(templates.Comment, Object.assign({}, props))));
    },
    Enum(props) {
        const { templates, reflection } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement(templates.Comment, Object.assign({}, props)),
            preact_1.createElement("table", { class: "enum" },
                preact_1.createElement("tbody", null, reflection.children.map((child) => (preact_1.createElement(templates.EnumMember, Object.assign({}, props, { reflection: child }))))))));
    },
    EnumMember(props) {
        const { templates, reflection } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement("tr", null,
                preact_1.createElement("td", null,
                    preact_1.createElement("p", null,
                        reflection.name,
                        " = ",
                        JSON.stringify(reflection.value))),
                preact_1.createElement("td", null,
                    preact_1.createElement(templates.Comment, Object.assign({}, props))))));
    },
    Function(props) {
        const { reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null, reflection.children.map((s) => {
            return preact_1.createElement(templates.Reflection, Object.assign({}, props, { reflection: s }));
        })));
    },
    Class(props) {
        const { templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement(templates.Comment, Object.assign({}, props)),
            preact_1.createElement(templates.PageChildren, Object.assign({}, props))));
    },
    Object(props) {
        const { reflection } = props;
        return preact_1.createElement(preact_1.Fragment, null,
            "TODO ",
            reflection.name);
    },
    Method(props) {
        const { reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null, reflection.children.map((s) => {
            return preact_1.createElement(templates.Reflection, Object.assign({}, props, { reflection: s }));
        })));
    },
    Property(props) {
        const { reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            reflection.isReadonly && "readonly ",
            reflection.name,
            ": ",
            preact_1.createElement(templates.Type, Object.assign({}, props, { type: reflection.type })),
            preact_1.createElement(templates.Comment, Object.assign({}, props))));
    },
    Accessor(props) {
        const { reflection, templates } = props;
        const type = reflection.hasGetter
            ? reflection.hasSetter
                ? "get/set"
                : "get"
            : "set";
        return (preact_1.createElement(preact_1.Fragment, null,
            type,
            " ",
            reflection.name,
            ":",
            " ",
            preact_1.createElement(templates.Type, Object.assign({}, props, { type: reflection.type })),
            preact_1.createElement(templates.Comment, Object.assign({}, props))));
    },
    Variable(props) {
        const { reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            reflection.name,
            ": ",
            preact_1.createElement(templates.Type, Object.assign({}, props, { type: reflection.type })),
            reflection.defaultValue && ` = ${reflection.defaultValue}`,
            preact_1.createElement(templates.Comment, Object.assign({}, props))));
    },
    Signature(props) {
        const { reflection, templates } = props;
        const typeArgs = (preact_1.createElement(templates.TypeParameters, Object.assign({}, props, { params: reflection.typeParameters })));
        const parameters = reflection.parameters.map((s, i) => {
            const rest = s.isRest ? "..." : "";
            const sep = s.isOptional ? "?: " : ": ";
            const init = s.defaultValue == null ? null : `= ${s.defaultValue}`;
            const comma = i === reflection.parameters.length - 1 ? null : ", ";
            return (preact_1.createElement(preact_1.Fragment, null,
                rest + s.name + sep,
                preact_1.createElement(templates.Type, Object.assign({}, props, { type: s.type })),
                init,
                comma));
        });
        const renderedParameters = reflection.parameters.length ? (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement("h4", null, "Parameters"),
            preact_1.createElement("ul", null, reflection.parameters.map((p) => (preact_1.createElement("li", null,
                preact_1.createElement(templates.Reflection, Object.assign({}, props, { reflection: p })))))))) : null;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement("div", { class: "signature" },
                reflection.name,
                typeArgs,
                "(",
                parameters,
                "):",
                " ",
                preact_1.createElement(templates.Type, Object.assign({}, props, { type: reflection.returnType }))),
            preact_1.createElement(templates.Comment, Object.assign({}, props)),
            renderedParameters));
    },
    Parameter(props) {
        const { reflection, templates } = props;
        return (preact_1.createElement(preact_1.Fragment, null,
            preact_1.createElement("b", null, reflection.name),
            ":",
            " ",
            preact_1.createElement(templates.Type, Object.assign({ type: reflection.type }, props)),
            preact_1.createElement("br", null),
            preact_1.createElement(templates.Comment, Object.assign({}, props))));
    },
    Reference({ reflection, router }) {
        const referenced = reflection.resolve();
        if (referenced) {
            return (preact_1.createElement(preact_1.Fragment, null,
                "Re-exports",
                " ",
                preact_1.createElement("a", { href: router.createLink(reflection, referenced) }, referenced.name)));
        }
        return preact_1.createElement(preact_1.Fragment, null,
            "Re-exports ",
            reflection.name);
    },
    Type(props) {
        const { reflection, router, type } = props;
        if ("id" in props.type) {
            return preact_1.createElement(props.templates.Object, Object.assign({}, props, { reflection: props.type }));
        }
        const linkTargets = [];
        const template = type_templates_1.TypeTemplates[props.type.kind];
        const typeString = template({
            ...props,
            type: type,
            linkTargets,
        });
        // Rather arbitrary cutoff here. With inferred types, the resulting type can be HUGE.
        // If we give Shiki a type which is too large, it will take an incredibly long time to
        // highlight... so disable highlighting for those types. See GH#1346 for a motivating example
        // where the inferred type results in a string >1MB in size.
        if (typeString.length > 200) {
            return preact_1.createElement(preact_1.Fragment, null, typeString);
        }
        // Doesn't matter if we use ts/tsx here. There are no type assertions or
        // TSX elements within types. For now, the template will put all output on one line,
        // so we can just grab the first line.
        const tokens = props.highlighter.getTokens(`type _ = ${typeString}`, "ts")[0];
        tokens.splice(0, tokens.findIndex((t) => /^\s*=/.test(t.content)));
        if (/^\s*=\s*$/.test(tokens[0].content)) {
            tokens.splice(0, 1);
        }
        else {
            tokens[0].content = tokens[0].content.replace(/^\s*=\s*/, "");
        }
        let targetIndex = 0;
        return (preact_1.createElement(preact_1.Fragment, null, tokens.map((token) => {
            const target = linkTargets[targetIndex];
            if (target instanceof models_1.Reflection && token.content === target.name) {
                targetIndex++;
                return (preact_1.createElement("a", { href: router.createLink(reflection, target), class: token.class }, token.content));
            }
            else if (target === token.content) {
                targetIndex++;
                return preact_1.createElement("span", { class: token.class }, token.content);
            }
            return preact_1.createElement("span", { class: token.class }, token.content);
        })));
    },
    TypeParameters(props) {
        const { params, templates } = props;
        if (params.length === 0) {
            return preact_1.createElement(preact_1.Fragment, null);
        }
        const children = ["<"];
        for (const param of params) {
            if (children.length !== 1) {
                children.push(", ");
            }
            children.push(param.name);
            if (param.constraint) {
                children.push(" extends ");
                children.push(preact_1.createElement(templates.Type, Object.assign({}, props, { type: param.constraint })));
            }
            if (param.defaultValue) {
                children.push(" = ");
                children.push(preact_1.createElement(templates.Type, Object.assign({}, props, { type: param.defaultValue })));
            }
        }
        children.push(">");
        return preact_1.createElement(preact_1.Fragment, null, children);
    },
};
//# sourceMappingURL=default-templates.js.map