"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DoubleHighlighter = void 0;
const assert_1 = require("assert");
const shiki_1 = require("shiki");
const preact_1 = require("preact");
const preact_render_to_string_1 = require("preact-render-to-string");
const Color = require("color");
const supportedLanguages = new Set([
    "ts",
    "typescript",
    "tsx",
    "js",
    "javascript",
]);
class DoubleHighlighter {
    constructor(light, lightBg, dark, darkBg) {
        this.light = light;
        this.lightBg = lightBg;
        this.dark = dark;
        this.darkBg = darkBg;
        // light | dark => class
        this.schemes = new Map();
        // class => [light, dark]
        this.reverseSchemes = new Map();
    }
    static async create(lightTheme, darkTheme) {
        const light = shiki_1.getTheme(lightTheme).bg;
        const dark = shiki_1.getTheme(darkTheme).bg;
        const [lightHl, darkHl] = await Promise.all([
            shiki_1.getHighlighter({ theme: lightTheme }),
            shiki_1.getHighlighter({ theme: darkTheme }),
        ]);
        return new DoubleHighlighter(lightHl, light, darkHl, dark);
    }
    highlight(code, lang) {
        // We don't know what this language is... just return the text,
        // escaped since code blocks ought not hold HTML.
        if (!supportedLanguages.has(lang)) {
            return preact_render_to_string_1.render(preact_1.createElement(preact_1.Fragment, null, code));
        }
        const children = [];
        for (const line of this.getTokens(code, lang)) {
            for (const token of line) {
                children.push(preact_1.createElement("span", { class: token.class }, token.content));
            }
            children.push(preact_1.createElement("br", null));
        }
        return preact_render_to_string_1.render(preact_1.createElement(preact_1.Fragment, null, children));
    }
    /**
     * Alternative version of {@link highlight} which can be used with extra information is needed
     * rather than directly rendering to a string.
     * @param code
     * @param lang
     */
    getTokens(code, lang) {
        assert_1.ok(supportedLanguages.has(lang));
        const lightTokens = this.light.codeToThemedTokens(code, lang, {
            includeExplanation: false,
        });
        const darkTokens = this.dark.codeToThemedTokens(code, lang, {
            includeExplanation: false,
        });
        // If this fails... something went *very* wrong.
        assert_1.ok(lightTokens.length === darkTokens.length);
        const codeTokens = [];
        for (let line = 0; line < lightTokens.length; line++) {
            const lightLine = lightTokens[line];
            const darkLine = darkTokens[line];
            // Different themes can have different grammars... so unfortunately we have to deal with different
            // sets of tokens.Example: light_plus and dark_plus tokenize " = " differently in the `schemes`
            // declaration for this file.
            const lineTokens = [];
            while (lightLine.length && darkLine.length) {
                // Simple case, same token.
                if (lightLine[0].content.length === darkLine[0].content.length) {
                    lineTokens.push({
                        class: this.getClass(lightLine[0].color, darkLine[0].color),
                        content: lightLine[0].content,
                    });
                    lightLine.shift();
                    darkLine.shift();
                    continue;
                }
                if (lightLine[0].content.length < darkLine[0].content.length) {
                    lineTokens.push({
                        class: this.getClass(lightLine[0].color, darkLine[0].color),
                        content: lightLine[0].content,
                    });
                    darkLine[0].content = darkLine[0].content.substr(lightLine[0].content.length);
                    lightLine.shift();
                    continue;
                }
                lineTokens.push({
                    class: this.getClass(lightLine[0].color, darkLine[0].color),
                    content: darkLine[0].content,
                });
                lightLine[0].content = lightLine[0].content.substr(darkLine[0].content.length);
                darkLine.shift();
            }
            codeTokens.push(lineTokens);
        }
        return codeTokens;
    }
    getStyles() {
        let styles = Array.from(this.reverseSchemes.values(), ([light, dark], i) => {
            return [
                `.hl-${i} { color: ${light}; }`,
                `.dark .hl-${i} { color: ${dark}; }`,
            ].join("\n");
        }).join("\n") + "\n";
        const lightBg = Color(this.lightBg);
        const darkBg = Color(this.darkBg);
        styles += `:root {
    --light-bg-hue: ${lightBg.hue()}deg;
    --dark-bg-hue: ${darkBg.hue()}deg;
    --light-bg-saturation: ${lightBg.saturationl()}%;
    --dark-bg-saturation: ${darkBg.saturationl()}%;
    --light-bg-lightness: ${lightBg.lightness()}%;
    --dark-bg-lightness: ${darkBg.lightness()}%;
    --light-bg-alpha: ${lightBg.alpha()};
    --dark-bg-alpha: ${darkBg.alpha()};
}\n`;
        return styles;
    }
    getClass(lightColor, darkColor) {
        // Rather arbitrary key, uses a separator that shouldn't ever appear in colors.
        const key = `${lightColor} | ${darkColor}`;
        let scheme = this.schemes.get(key);
        if (scheme == null) {
            scheme = `hl-${this.schemes.size}`;
            this.schemes.set(key, scheme);
            this.reverseSchemes.set(scheme, [lightColor, darkColor]);
        }
        return scheme;
    }
}
exports.DoubleHighlighter = DoubleHighlighter;
//# sourceMappingURL=highlight.js.map