import type { VNode } from "preact";
import type { Application } from "../application";
import type { ProjectReflection } from "../models";
import { DeepPartial, EventHooks, Keys, StringIfExternal } from "../utils";
import { DoubleHighlighter } from "./highlight";
import type { ThemeRouter, ThemeRouterConstructor } from "./router";
import type { TemplateHooks, Templates } from "./templates";
import { defaultThemeDefinition } from "./theme";
export declare type TypeDocThemes = "default" | "minimal";
export interface ThemeContext {
    highlighter: DoubleHighlighter;
    router: ThemeRouter;
    hooks: EventHooks<TemplateHooks, VNode | null>;
    outDir: string;
    app: Application;
}
export declare type BuiltinThemeActions = Keys<typeof defaultThemeDefinition["preRender" | "postRender"]>;
export interface ThemeDefinition {
    /**
     * The router used by this theme. If not set, will use default {@link ThemeRouter}.
     */
    readonly router: ThemeRouterConstructor;
    /**
     * The templates used by this theme. If not set, will use the default templates.
     */
    readonly templates: Templates;
    /**
     * Actions performed before rendering the project with this theme's templates.
     */
    readonly preRender: Record<string, (context: ThemeContext) => Promise<void> | void>;
    /**
     * Actions performed after rendering the project with this theme's templates.
     */
    readonly postRender: Record<string, (context: ThemeContext) => Promise<void> | void>;
}
export interface ThemeConfiguration extends DeepPartial<ThemeDefinition> {
    /**
     * An optional theme to inherit settings from.
     */
    readonly parent?: StringIfExternal<TypeDocThemes>;
    /**
     * Any pre-render or post-render actions which need to be disabled by this theme.
     */
    readonly suppress?: readonly StringIfExternal<BuiltinThemeActions>[];
}
export declare class Renderer {
    readonly application: Application;
    private _themes;
    /**
     * Hooks which plugins can use to modify the rendered output.
     * Any hooks added during a render will only be retained for that render call.
     */
    readonly hooks: EventHooks<TemplateHooks, VNode<{}> | null>;
    constructor(application: Application);
    private get logger();
    /**
     * Renders the given project using the currently selected theme.
     * @param project
     * @param out
     */
    render(project: ProjectReflection, out: string): Promise<void>;
    /**
     * Adds a theme to TypeDoc that can be specified with `--theme`.
     * @param name
     * @param theme
     */
    addTheme(name: string, theme: ThemeConfiguration): void;
    /**
     * Gets a defined theme, throws if it does not exist.
     *
     * Note: `string & {}` is used here to trick TS into providing autocomplete for
     * the builtin actions while also allowing any string.
     */
    private getTheme;
}
