"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Renderer = void 0;
const assert_1 = require("assert");
const path_1 = require("path");
const preact_1 = require("preact");
const preact_render_to_string_1 = require("preact-render-to-string");
const utils_1 = require("../utils");
const array_1 = require("../utils/array");
const fs_1 = require("../utils/fs");
const comment_1 = require("./comment");
const highlight_1 = require("./highlight");
const theme_1 = require("./theme");
class Renderer {
    constructor(application) {
        this.application = application;
        this._themes = new Map();
        /**
         * Hooks which plugins can use to modify the rendered output.
         * Any hooks added during a render will only be retained for that render call.
         */
        this.hooks = new utils_1.EventHooks();
        this._themes.set("default", theme_1.defaultThemeDefinition);
        this.addTheme("minimal", theme_1.minimalThemeDefinition);
    }
    get logger() {
        return this.application.logger;
    }
    /**
     * Renders the given project using the currently selected theme.
     * @param project
     * @param out
     */
    async render(project, out) {
        const start = Date.now();
        const theme = this.getTheme(this.application.options.getValue("theme"));
        const momento = this.hooks.saveMomento();
        if (this.application.options.getValue("cleanOutputDir")) {
            await utils_1.remove(out);
        }
        const router = new theme.router(project);
        // TODO: These ought to be configurable.
        const highlighter = await highlight_1.DoubleHighlighter.create("light-plus", "dark-plus");
        const context = {
            highlighter,
            router,
            hooks: this.hooks,
            outDir: out,
            app: this.application,
        };
        // It seems like this ought to live elsewhere...
        const boundParseMarkdown = (markdown, reflection) => {
            let result = comment_1.replaceIncludes(this.application.options.getValue("includes"), markdown, this.application.logger);
            result = comment_1.replaceMedia(this.application.options.getValue("media"), result, this.application.logger, reflection, router);
            return comment_1.parseMarkdown(markdown, reflection, router, highlighter);
        };
        this.logger.verbose(`Running pre-render actions: ${Object.keys(theme.preRender).join(", ") || "none"}`);
        const beforePreRender = Date.now();
        await array_1.waterfall(Object.values(theme.preRender), (action) => action(context));
        const afterPre = Date.now();
        this.logger.verbose(`[Perf] Pre-render actions took ${afterPre - beforePreRender}ms`);
        const pages = [project];
        const tasks = [];
        let page = pages.shift();
        while (page) {
            assert_1.ok(router.hasOwnDocument(page));
            const path = router.getDocumentName(page);
            this.logger.verbose(`${page.getFullName()} ===> ${path}`);
            const content = "<!DOCTYPE html>\n" +
                preact_render_to_string_1.render(preact_1.createElement(theme.templates.Page, { hooks: this.hooks, reflection: page, router: router, parseMarkdown: boundParseMarkdown, templates: theme.templates, highlighter: highlighter }), null, { pretty: false });
            tasks.push(fs_1.writeFile(path_1.join(out, path), content));
            if (page.isContainer()) {
                pages.push(...page.children.filter((child) => router.hasOwnDocument(child)));
            }
            page = pages.shift();
        }
        const afterRender = Date.now();
        this.logger.verbose(`[Perf] Template rendering took ${afterRender - afterPre}ms`);
        this.logger.verbose(`Running post-render actions: ${Object.keys(theme.postRender).join(", ") || "none"}`);
        await array_1.waterfall(Object.values(theme.postRender), (action) => action(context));
        const afterPost = Date.now();
        this.logger.verbose(`[Perf] Post-render actions took ${afterPost - afterRender}ms`);
        this.hooks.restoreMomento(momento);
        this.logger.verbose(`[Perf] Full render took ${afterPost - start}ms`);
    }
    /**
     * Adds a theme to TypeDoc that can be specified with `--theme`.
     * @param name
     * @param theme
     */
    addTheme(name, theme) {
        var _a;
        if (this._themes.has(name)) {
            throw new Error(`The theme "${name}" has already been defined.`);
        }
        const definition = cloneDefinition(theme.parent ? this.getTheme(theme.parent) : theme_1.defaultThemeDefinition);
        if (theme.router) {
            definition.router = theme.router;
        }
        if (theme.templates) {
            Object.assign(definition.templates, theme.templates);
        }
        if (theme.preRender) {
            Object.assign(definition.preRender, theme.preRender);
        }
        if (theme.postRender) {
            Object.assign(definition.postRender, theme.postRender);
        }
        for (const item in (_a = theme.suppress) !== null && _a !== void 0 ? _a : []) {
            delete definition.preRender[item];
            delete definition.postRender[item];
        }
        this._themes.set(name, definition);
    }
    /**
     * Gets a defined theme, throws if it does not exist.
     *
     * Note: `string & {}` is used here to trick TS into providing autocomplete for
     * the builtin actions while also allowing any string.
     */
    getTheme(name) {
        const theme = this._themes.get(name);
        if (!theme) {
            throw new Error(`The theme "${name}" has not been defined`);
        }
        return theme;
    }
}
exports.Renderer = Renderer;
// No need to pull in a new package for deep cloning... we only need one level.
function cloneDefinition(def) {
    return {
        router: def.router,
        templates: { ...def.templates },
        preRender: { ...def.preRender },
        postRender: { ...def.postRender },
    };
}
//# sourceMappingURL=renderer.js.map