import { Slugger } from "marked";
import { ProjectReflection, Reflection, SomeReflection } from "../models";
/**
 * Type which custom routers must conform to.
 */
export declare type ThemeRouterConstructor = new (project: ProjectReflection) => ThemeRouter;
/**
 * Defines how to resolve links between reflections and which reflections get their own page.
 */
export declare class ThemeRouter {
    protected project: ProjectReflection;
    /**
     * A map of reflections with pages to their associated slugger instance.
     */
    protected _sluggers: WeakMap<Reflection, Slugger>;
    protected _slugs: WeakMap<Slugger, WeakMap<Reflection, string>>;
    constructor(project: ProjectReflection);
    /**
     * Create a link from `from` to `to`, including an anchor if required.
     */
    createLink(from: Reflection, to: Reflection): string;
    /**
     * Create a unique anchor for the given reflection or header.
     * Note: Anchors must be case-insensitive unique from all other anchors in the page.
     * This requires that any rendered markdown also use this method to generate a slug.
     * This is correctly handled by the slugger provided by Marked, which is used internally.
     * @param reflection
     */
    createSlug(reflection: Reflection, header?: string): string;
    /**
     * Creates a link from the page for `from` to the specified asset.
     */
    createAssetLink(from: Reflection, asset: string): string;
    /**
     * Get the directory to store assets in relative to the root theme directory.
     * Used by {@link createAssetLink} to determine how to link to an asset.
     */
    getAssetDirectory(): string;
    /**
     * Creates a link from the page for `from` to the specified asset.
     */
    createMediaLink(from: Reflection, media: string): string;
    /**
     * Get the directory to store media in relative to the root theme directory.
     * Used by {@link createMediaLink} to determine how to link to an asset.
     */
    getMediaDirectory(): string;
    /**
     * Used to get the file name to be written to the file system, may be used
     * even if a reflection does not own its own page.
     *
     * Produces filenames like:
     * mod/ns/class.Foo.html for a class Foo within namespace ns within module mod
     * or
     * mod/ns/index.html for a namespace ns within module mod
     * The kind string in the final filename is important. Without it, reflections which are
     * merged would conflict.
     * ```ts
     * interface Foo {}
     * namespace Foo {} // legal!
     * ```
     *
     * Note: Windows doesn't care if we use / to separate directories, so we can use / here and use
     * the generated names both on the web and to write files.
     *
     * @param reflection
     */
    getDocumentName(reflection: Reflection): string;
    /**
     * Used to determine if a reflection should get its own page, or be contained within its parent's page.
     *
     * Rules:
     * 1. If `false` is returned for a reflection, all children reflections must also return `false`.
     * 2. If `true` is returned for a reflection, its children may return `true` or `false`.
     * 3. As project reflections are the root level reflection, this must return true if `reflection.isProject()` is true.
     */
    hasOwnDocument(reflection: Reflection): boolean;
    /**
     * Helper function for templates to get direct children which should be rendered in this page.
     * @param reflection
     */
    getChildrenInPage(reflection: Reflection): SomeReflection[];
    /**
     * Helper function to find the reflection whose page this reflection will be contained in.
     */
    protected _getReflectionWithDocument(reflection: Reflection): Reflection;
    /**
     * Transform the given filename into a safe filename, replacing special characters.
     */
    protected _getSafeFilename(name: string): string;
    /**
     * Helper to determine if the given reflection is going to be a directory.
     */
    protected _anyChildrenHaveOwnDocument(reflection: Reflection): boolean;
}
export declare class MinimalThemeRouter extends ThemeRouter {
    hasOwnDocument(reflection: Reflection): boolean;
}
