"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeTemplates = void 0;
const models_1 = require("../models");
const mapped_1 = require("../models/types/mapped");
function wrap(wrapped, text) {
    if (wrapped) {
        return `(${text})`;
    }
    return text;
}
function toString(props) {
    // Cast is necessary due to discriminated unions not playing well with an intersection.
    return exports.TypeTemplates[props.type.kind](props);
}
exports.TypeTemplates = {
    [models_1.TypeKind.Array](props) {
        return (toString({ ...props, type: props.type.elementType, wrapped: true }) + "[]");
    },
    [models_1.TypeKind.Conditional](props) {
        const { type, wrapped } = props;
        return wrap(wrapped, toString({ ...props, wrapped: true, type: type.checkType }) +
            " extends " +
            toString({ ...props, wrapped: true, type: type.extendsType }) +
            " ? " +
            toString({ ...props, wrapped: true, type: type.trueType }) +
            " : " +
            toString({ ...props, wrapped: true, type: type.falseType }));
    },
    [models_1.TypeKind.IndexedAccess](props) {
        const { type } = props;
        return (toString({ ...props, type: type.objectType, wrapped: true }) +
            "[" +
            toString({ ...props, type: type.indexType, wrapped: false }) +
            "]");
    },
    [models_1.TypeKind.Inferred](props) {
        return wrap(props.wrapped, `infer ${props.type.name}`);
    },
    [models_1.TypeKind.Intersection](props) {
        const { type } = props;
        const children = type.types.map((type) => toString({ ...props, type, wrapped: true }));
        return wrap(props.wrapped, children.join(" & "));
    },
    [models_1.TypeKind.Intrinsic](props) {
        return props.type.name;
    },
    [models_1.TypeKind.Literal](props) {
        const { type } = props;
        if (typeof type.value === "object" && type.value) {
            return `${type.value.negative ? "-" : ""}${type.value.value}n`;
        }
        return JSON.stringify(type.value);
    },
    [models_1.TypeKind.Mapped](props) {
        const { type, linkTargets } = props;
        linkTargets.push(type.parameter.name);
        const children = [
            "{",
            {
                [mapped_1.OptionalModifier.None]: " ",
                [mapped_1.OptionalModifier.Add]: " readonly ",
                [mapped_1.OptionalModifier.Remove]: " -readonly ",
            }[type.readonlyModifier],
            `[${type.parameter.name} in `,
            toString({ ...props, type: type.parameter.constraint }),
            "]",
            {
                [mapped_1.OptionalModifier.None]: ": ",
                [mapped_1.OptionalModifier.Add]: "?: ",
                [mapped_1.OptionalModifier.Remove]: "-?: ",
            }[type.optionalModifier],
            toString({ ...props, type: type.type, wrapped: false }),
            " }",
        ];
        return children.join("");
    },
    [models_1.TypeKind.Object](props) {
        const { type } = props;
        const members = [
            ...type.properties.map((prop) => toString({ ...props, type: prop })),
            ...type.signatures.map((prop) => toString({ ...props, type: prop })),
            ...type.constructSignatures.map((prop) => toString({ ...props, type: prop })),
        ];
        return `{ ${members.join(", ")} }`;
    },
    [models_1.TypeKind.Property](props) {
        const { type } = props;
        const children = [];
        if (type.isReadonly) {
            children.push("readonly ");
        }
        // Checking if it is a valid identifier & thus doesn't need to be quoted is... ridiculous.
        // Instead, use an intentionally simplistic check. https://stackoverflow.com/a/9337047/7186598
        if (/^[A-Za-z0-9_$]+$/.test(type.name)) {
            children.push(type.name);
        }
        else {
            children.push(JSON.stringify(type.name));
        }
        children.push(type.isOptional ? "?: " : ": ");
        children.push(toString({ ...props, type: type.propertyType, wrapped: false }));
        return children.join("");
    },
    [models_1.TypeKind.Predicate](props) {
        const { type, linkTargets } = props;
        linkTargets.push(type.name);
        const children = type.asserts ? ["asserts ", type.name] : [type.name];
        if (type.targetType) {
            children.push(" is ", toString({ ...props, type: type.targetType }));
        }
        return children.join("");
    },
    [models_1.TypeKind.Query](props) {
        const { type } = props;
        return `typeof ${toString({ ...props, type: type.queryType })}`;
    },
    [models_1.TypeKind.Reference](props) {
        var _a, _b, _c;
        const { type, linkTargets } = props;
        linkTargets.push((_a = type.reflection) !== null && _a !== void 0 ? _a : type.name);
        let typeArgs = type.typeArguments
            .map((arg) => toString({ ...props, type: arg, wrapped: false }))
            .join(", ");
        if (typeArgs.length) {
            typeArgs = `<${typeArgs}>`;
        }
        return `${(_c = (_b = type.reflection) === null || _b === void 0 ? void 0 : _b.name) !== null && _c !== void 0 ? _c : type.name}${typeArgs}`;
    },
    [models_1.TypeKind.Signature](props) {
        const { type } = props;
        const parameters = type.parameters
            .map((param) => toString({ ...props, type: param, wrapped: false }))
            .join(", ");
        let typeParameters = type.typeParameters
            .map((param) => toString({ ...props, type: param, wrapped: false }))
            .join(", ");
        if (typeParameters.length) {
            typeParameters = `<${typeParameters}>`;
        }
        return wrap(props.wrapped, `${typeParameters}(${parameters}): ${toString({
            ...props,
            type: type.returnType,
        })}`);
    },
    [models_1.TypeKind.Constructor](props) {
        const { type } = props;
        const parameters = type.parameters
            .map((param) => toString({ ...props, type: param, wrapped: false }))
            .join(", ");
        let typeParameters = type.typeParameters
            .map((param) => toString({ ...props, type: param, wrapped: false }))
            .join(", ");
        if (typeParameters.length) {
            typeParameters = `<${typeParameters}>`;
        }
        return wrap(props.wrapped, `new ${typeParameters}(${parameters}): ${toString({
            ...props,
            type: type.returnType,
            wrapped: false,
        })}`);
    },
    [models_1.TypeKind.SignatureParameter](props) {
        const { type, linkTargets } = props;
        const children = [];
        if (type.isRest) {
            children.push("...");
        }
        linkTargets.push(type.name);
        children.push(type.name, type.isOptional ? "?: " : ": ");
        children.push(toString({ ...props, type: type.parameterType }));
        return children.join("");
    },
    [models_1.TypeKind.Tuple](props) {
        const { type } = props;
        return `[${type.elements
            .map((type) => toString({ ...props, type }))
            .join(", ")}]`;
    },
    [models_1.TypeKind.TupleMember](props) {
        const { type, linkTargets } = props;
        linkTargets.push(type.name);
        const opt = type.isOptional ? "?" : "";
        return `${type.name}${opt}: ${toString({ ...props, type: type.type })}`;
    },
    [models_1.TypeKind.TypeOperator](props) {
        const { type } = props;
        return `${type.operator} ${toString({ ...props, type: type.target })}`;
    },
    [models_1.TypeKind.TypeParameter](props) {
        const { type, linkTargets } = props;
        linkTargets.push(type.name);
        const children = [type.name];
        if (type.constraint) {
            children.push(" extends ", toString({ ...props, type: type.constraint }));
        }
        if (type.defaultValue) {
            children.push(" = ", toString({ ...props, type: type.defaultValue }));
        }
        return children.join("");
    },
    [models_1.TypeKind.Union](props) {
        const { type } = props;
        const children = type.types.map((type) => toString({ ...props, type, wrapped: true }));
        return wrap(props.wrapped, children.join(" | "));
    },
    [models_1.TypeKind.Unknown](props) {
        // Ideally we should never run into this code. If we do, we might have broken links
        // in the type displayed after this type...
        return props.type.name;
    },
    [models_1.TypeKind.Optional](props) {
        return wrap(props.wrapped, toString({ ...props, type: props.type.type, wrapped: true }) + "?");
    },
};
//# sourceMappingURL=type-templates.js.map