"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Serializer = void 0;
const assert = require("assert");
const models_1 = require("../models");
const utils_1 = require("../utils");
class Serializer {
    constructor() {
        this._reflectionSerializers = new Map();
        this._typeSerializers = new Map();
        addBaseSerializers(this);
    }
    addReflectionSerializer(kinds, worker) {
        var _a;
        for (const kind of models_1.ReflectionKind.toKindArray(kinds)) {
            const group = (_a = this._reflectionSerializers.get(kind)) !== null && _a !== void 0 ? _a : [];
            utils_1.insertOrderSorted(group, worker);
            this._reflectionSerializers.set(kind, group);
        }
    }
    addTypeSerializer(kinds, worker) {
        var _a;
        for (const kind of models_1.TypeKind.toKindArray(kinds)) {
            const group = (_a = this._typeSerializers.get(kind)) !== null && _a !== void 0 ? _a : [];
            utils_1.insertOrderSorted(group, worker);
            this._typeSerializers.set(kind, group);
        }
    }
    toObject(value, init = {}) {
        let serializers;
        if (value instanceof models_1.Reflection) {
            serializers = this._reflectionSerializers.get(value.kind);
        }
        else if (value instanceof models_1.Type) {
            serializers = this._typeSerializers.get(value.kind);
        }
        assert(serializers, `No serializers registered for object with kind: ${value.kind}. This is a bug.`);
        return serializers.reduce((serialized, worker) => worker.serialize(value, serialized), init);
    }
    toObjects(values) {
        return values.map((value) => this.toObject(value));
    }
}
exports.Serializer = Serializer;
function addBaseSerializers(serializer) {
    serializer.addTypeSerializer(models_1.TypeKind.All, {
        order: -1,
        serialize(type, init) {
            const base = {
                ...init,
                kind: type.kind,
                kindString: models_1.TypeKind.toKindString(type.kind),
            };
            // TS can't validate that `base` is the right type.
            return type.serialize(serializer, base);
        },
    });
    serializer.addReflectionSerializer(models_1.ReflectionKind.All, {
        order: -1,
        serialize(reflection, init) {
            var _a, _b;
            const base = {
                ...init,
                id: reflection.id,
                name: reflection.name,
                kind: reflection.kind,
                kindString: models_1.ReflectionKind.toKindString(reflection.kind),
            };
            if (reflection.originalName !== reflection.name) {
                base.originalName = reflection.originalName;
            }
            if (reflection.comment) {
                base.comment = {
                    shortText: reflection.comment.shortText,
                    text: reflection.comment.text,
                    tags: (_b = (_a = reflection.comment.tags) === null || _a === void 0 ? void 0 : _a.map((tag) => ({
                        tagName: tag.tagName,
                        text: tag.text,
                        paramName: tag.paramName,
                    }))) !== null && _b !== void 0 ? _b : [],
                };
            }
            if (reflection.isContainer()) {
                const children = serializer.toObjects(reflection.children);
                // TS isn't quite smart enough to know this is OK.
                base.children = children;
            }
            // TS can't validate base is the right type here.
            return reflection.serialize(serializer, base);
        },
    });
}
//# sourceMappingURL=serializer.js.map