"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterMap = exports.waterfall = exports.zip = exports.uniq = exports.removeIfPresent = exports.binaryFindPartition = exports.insertOrderSorted = void 0;
const assert_1 = require("assert");
/**
 * Inserts an item into an array sorted by order. If two items have the same order,
 * the item inserted later will be placed later in the array.
 * @param arr modified by inserting item.
 * @param item
 */
function insertOrderSorted(arr, item) {
    const index = binaryFindPartition(arr, (v) => { var _a, _b; return ((_a = v.order) !== null && _a !== void 0 ? _a : 0) > ((_b = item.order) !== null && _b !== void 0 ? _b : 0); });
    arr.splice(index === -1 ? arr.length : index, 0, item);
    return arr;
}
exports.insertOrderSorted = insertOrderSorted;
/**
 * Performs a binary search of a given array, returning the index of the first item
 * for which `partition` returns true. Returns the -1 if there are no items in `arr`
 * such that `partition(item)` is true.
 * @param arr
 * @param partition should return true while less than the partition point.
 */
function binaryFindPartition(arr, partition) {
    if (arr.length === 0) {
        return -1;
    }
    let low = 0, high = arr.length - 1;
    while (high > low) {
        const mid = low + Math.floor((high - low) / 2);
        if (partition(arr[mid])) {
            high = mid;
        }
        else {
            low = mid + 1;
        }
    }
    return partition(arr[low]) ? low : -1;
}
exports.binaryFindPartition = binaryFindPartition;
/**
 * Removes an item from the array if the array exists and the item is included
 * within it.
 * @param arr
 * @param item
 */
function removeIfPresent(arr, item) {
    if (!arr) {
        return;
    }
    const index = arr.indexOf(item);
    if (index !== -1) {
        arr.splice(index, 1);
    }
}
exports.removeIfPresent = removeIfPresent;
/**
 * Filters out duplicate values from the array. Order is preserved. Returns
 * a copy of the input array.
 */
function uniq(arr) {
    return Array.from(new Set(arr));
}
exports.uniq = uniq;
/**
 * Standard zip function, expects that all arrays have the same length.
 */
function zip(...arrays) {
    const result = [];
    const length = Math.min(...arrays.map((a) => a.length));
    assert_1.ok(Math.max(...arrays.map((a) => a.length)) === length, "zip arrays do not have the same length");
    for (let i = 0; i < length; i++) {
        result.push(arrays.map((a) => a[i]));
    }
    return result;
}
exports.zip = zip;
async function waterfall(arr, fn) {
    const result = [];
    for (const item of arr) {
        result.push(await fn(item));
    }
    return result;
}
exports.waterfall = waterfall;
function filterMap(arr, mapper) {
    const result = [];
    for (const item of arr) {
        const mapped = mapper(item);
        if (mapped != null) {
            result.push(mapped);
        }
    }
    return result;
}
exports.filterMap = filterMap;
//# sourceMappingURL=array.js.map