/**
 * Simple, type safe, async event emitter class.
 *
 * @example
 * ```ts
 * const x = new EventEmitter<{ a: [string] }>()
 * x.on('a', a => a.repeat(123)) // ok
 * x.on('b', console.log) // error, 'b' is not assignable to 'a'
 * const y = new EventEmitter<{ a: [string]; [k: string]: unknown[] }>()
 * y.on('a', a => a.repeat(123)) // ok
 * // ok, any unknown events will contain an unknown number of arguments.
 * y.on('b', (...args: unknown[]) => console.log(...args))
 * ```
 */
export declare class EventEmitter<T extends Record<keyof T, unknown[]>> {
    private _listeners;
    /**
     * Starts listening to an event.
     * @param event the event to listen to.
     * @param listener function to be called when an this event is emitted.
     */
    on<K extends keyof T>(event: K, listener: (...args: T[K]) => void | Promise<void>): void;
    /**
     * Listens to a single occurrence of an event.
     * @param event the event to listen to.
     * @param listener function to be called when an this event is emitted.
     */
    once<K extends keyof T>(event: K, listener: (...args: T[K]) => void | Promise<void>): void;
    /**
     * Stops listening to an event.
     * @param event the event to stop listening to.
     * @param listener the function to remove from the listener array.
     */
    off<K extends keyof T>(event: K, listener: (...args: T[K]) => void | Promise<void>): void;
    /**
     * Emits an event to all currently subscribed listeners.
     * @param event the event to emit.
     * @param args any arguments required for the event.
     */
    emit<K extends keyof T>(event: K, ...args: T[K]): Promise<void>;
}
