"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.remove = exports.copy = exports.copyFile = exports.writeFile = exports.expandDirectories = exports.getCommonDirectory = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const paths_1 = require("./paths");
/**
 * Get the longest directory path common to all files.
 *
 * Used to infer a root directory for a project based on the input files if `rootDir` is not set.
 * @param files
 */
function getCommonDirectory(files) {
    if (!files.length) {
        return "";
    }
    const roots = files.map((f) => f.split(/\\|\//));
    if (roots.length === 1) {
        return roots[0].slice(0, -1).join("/");
    }
    let i = 0;
    while (new Set(roots.map((part) => part[i])).size === 1) {
        i++;
    }
    return roots[0].slice(0, i).join("/");
}
exports.getCommonDirectory = getCommonDirectory;
/**
 * Expand a list of input files and directories to get input files for a program.
 *
 * @param entryPoint The list of files that should be expanded.
 * @param excludePatterns Patterns to test files not in the entryPoint parameter against for exclusion.
 * @param allowJs Whether or not to include JS files in the result.
 * @returns The list of input files with expanded directories.
 */
async function expandDirectories(entryPoint, excludePatterns, allowJs, logger) {
    const files = [];
    const exclude = paths_1.createMinimatch(excludePatterns);
    const supportedFileRegex = allowJs ? /\.[tj]sx?$/ : /\.tsx?$/;
    function isDeclarationFile(fileName) {
        return /\.d\.tsx?$/.test(fileName);
    }
    function isExcluded(fileName) {
        return exclude.some((mm) => mm.match(fileName));
    }
    async function add(file, entryPoint) {
        let stats;
        try {
            stats = await fs_1.promises.stat(file);
        }
        catch (_a) {
            // No permission or a symbolic link, do not resolve.
            if (entryPoint) {
                logger === null || logger === void 0 ? void 0 : logger.error(`Entry point ${file} does not exist or cannot be read.`);
            }
            return;
        }
        const fileIsDir = stats.isDirectory();
        if (fileIsDir && !file.endsWith("/")) {
            file = `${file}/`;
        }
        if (!entryPoint && isExcluded(file.replace(/\\/g, "/"))) {
            return;
        }
        // If someone gave us a directory with allowJs set, they probably have both js and d.ts
        // files. We should only expand the js files - the d.ts files will be pulled in by the
        // compiler as needed.
        if (!entryPoint && allowJs && isDeclarationFile(file)) {
            return;
        }
        if (fileIsDir) {
            const children = await fs_1.promises.readdir(file);
            await Promise.all(children.map((child) => add(path_1.join(file, child), false)));
        }
        else if (supportedFileRegex.test(file)) {
            files.push(file);
        }
    }
    await Promise.all(entryPoint.map((file) => add(path_1.resolve(file), true)));
    return files.sort();
}
exports.expandDirectories = expandDirectories;
async function writeFile(path, content) {
    await fs_1.promises.mkdir(path_1.dirname(path), { recursive: true });
    await fs_1.promises.writeFile(path, content);
}
exports.writeFile = writeFile;
async function copyFile(src, dest) {
    await fs_1.promises.mkdir(path_1.dirname(dest), { recursive: true });
    await fs_1.promises.copyFile(src, dest);
}
exports.copyFile = copyFile;
/**
 * Copy a file or directory recursively.
 */
async function copy(src, dest) {
    const stat = await fs_1.promises.stat(src);
    if (stat.isDirectory()) {
        const contained = await fs_1.promises.readdir(src);
        await Promise.all(contained.map((file) => copy(path_1.join(src, file), path_1.join(dest, file))));
    }
    else if (stat.isFile()) {
        await fs_1.promises.mkdir(path_1.dirname(dest), { recursive: true });
        await fs_1.promises.copyFile(src, dest);
    }
    else {
        // Do nothing for FIFO, special devices.
    }
}
exports.copy = copy;
/**
 * Async recursive rmdir. Node v12.10.0 adds the { recursive: true } option to
 * the native fs rmdir function, but we don't require Node 12 yet.
 * @param target
 */
async function remove(target) {
    let isFile;
    try {
        const stat = await fs_1.promises.lstat(target);
        isFile = !stat.isDirectory();
    }
    catch (_a) {
        return;
    }
    if (isFile) {
        return fs_1.promises.unlink(target);
    }
    else {
        const files = await fs_1.promises.readdir(target);
        await Promise.all(files.map((file) => remove(path_1.join(target, file))));
        return fs_1.promises.rmdir(target);
    }
}
exports.remove = remove;
//# sourceMappingURL=fs.js.map