import type * as ts from "typescript";
import { DeclarationOption, ParameterScope, TypeDocOptions, KeyToDeclaration, TypeDocAndTSOptions, TypeDocOptionMap, TypeDocOptionValues } from "./declaration";
import type { Logger } from "../loggers";
import type { Application } from "../../..";
import { NeverIfInternal } from "..";
/**
 * Describes an option reader that discovers user configuration and converts it to the
 * TypeDoc format.
 */
export interface OptionsReader {
    /**
     * Readers will be processed according to their order.
     * A higher order indicates that the reader should be called *later* so that
     * it can override options set by lower order readers.
     *
     * Note that to preserve expected behavior, the argv reader must have both the lowest
     * order so that it may set the location of config files used by other readers and
     * the highest order so that it can override settings from lower order readers.
     */
    order: number;
    /**
     * The name of this reader so that it may be removed by plugins without the plugin
     * accessing the instance performing the read. Multiple readers may have the same
     * name.
     */
    name: string;
    /**
     * Read options from the reader's source and place them in the options parameter.
     * Options without a declared name may be treated as if they were declared with type
     * [[ParameterType.Mixed]]. Options which have been declared must be converted to the
     * correct type. As an alternative to doing this conversion in the reader,
     * the reader may use [[Options.setValue]], which will correctly convert values.
     * @param options
     * @param compilerOptions
     * @param container the options container that provides declarations
     * @param logger
     */
    read(container: Options, logger: Logger): void;
}
/**
 * Maintains a collection of option declarations split into TypeDoc options
 * and TypeScript options. Ensures options are of the correct type for calling
 * code.
 *
 * ### Case Sensitivity
 * All option keys are case insensitive. The following lines will each get the same value.
 * ```ts
 * const x = options.getValue('name');
 * const y = options.getValue('NAME');
 * const z = options.tryGetValue('NaMe').unwrap();
 * ```
 *
 * **WARNING**: This case insensitivity is primarily intended to ease command line use and for
 * backward compatibility. It may change in a future release. Any code using TypeDoc options
 * programmatically should conform to the case indicated in the [[TypeDocOptions]] interface.
 *
 * ### Option Discovery
 *
 * Since plugins commonly add custom options, and TypeDoc does not permit options which have
 * not been declared to be set, options must be read twice. The first time options are read,
 * a noop logger is passed so that any errors are ignored. Then, after loading plugins, options
 * are read again, this time with the logger specified by the application.
 *
 * Options are read in a specific order.
 * 1. argv (0) - Must be read first since it should change the files read when
 *    passing --options or --tsconfig.
 * 2. typedoc-json (100) - Read next so that it can specify the tsconfig.json file to read.
 * 3. tsconfig-json (200) - Last config file reader, cannot specify the typedoc.json file to read.
 * 4. argv (300) - Read argv again since any options set there should override those set in config
 *    files.
 */
export declare class Options {
    private _readers;
    private _declarations;
    private _values;
    private _compilerOptions;
    private _logger;
    constructor(logger: Logger);
    /**
     * Sets the logger used when an option declaration fails to be added.
     * @param logger
     */
    setLogger(logger: Logger): void;
    /**
     * Adds the option declarations declared by the TypeDoc and all supported TypeScript declarations.
     */
    addDefaultDeclarations(): void;
    /**
     * Gets a help string to display to the user if the -h flag is passed.
     */
    getHelp(): string;
    /**
     * Resets the option bag to all default values.
     */
    reset(): void;
    /**
     * Adds an option reader that will be used to read configuration values
     * from the command line, configuration files, or other locations.
     * @param reader
     */
    addReader(reader: OptionsReader): void;
    /**
     * Removes all readers of a given name.
     * @param name
     */
    removeReaderByName(name: string): void;
    read(logger: Logger): void;
    /**
     * Adds an option declaration to the container with extra type checking to ensure that
     * the runtime type is consistent with the declared type.
     * @param declaration
     */
    addDeclaration<K extends keyof TypeDocOptions>(declaration: {
        name: K;
    } & KeyToDeclaration<K>): void;
    /**
     * Adds an option declaration to the container.
     * @param declaration
     */
    addDeclaration(declaration: NeverIfInternal<Readonly<DeclarationOption>>): void;
    /**
     * Adds the given declarations to the container
     * @param declarations
     *
     * @privateRemarks
     * This function explicitly provides a way out of the strict typing enforced
     * by {@link addDeclaration}. It should only be used with careful validation
     * of the declaration map. Internally, it is only used for adding TS options
     */
    addDeclarations(declarations: readonly DeclarationOption[]): void;
    /**
     * Removes a declared option.
     * WARNING: This is probably a bad idea. If you do this you will probably cause a crash
     * when code assumes that an option that it declared still exists.
     * @param name
     */
    removeDeclarationByName(name: string): void;
    /**
     * Gets a declaration by either its name or short name.
     * @param name
     */
    getDeclaration(name: string): Readonly<DeclarationOption> | undefined;
    /**
     * Gets all declarations in the options with a given scope.
     * @param scope
     */
    getDeclarationsByScope(scope: ParameterScope): DeclarationOption[];
    /**
     * Checks if the given option has a set value or if the value is the default value.
     * @param name
     */
    isDefault(name: keyof TypeDocOptions): boolean;
    isDefault(name: NeverIfInternal<string>): boolean;
    /**
     * Gets all of the TypeDoc option values defined in this option container.
     * Note that this is a *shallow* clone of the internally defined options. If a plugin defines
     * object options which contain mutable properties, it is their responsibility to correctly
     * handle mutability.
     */
    getRawValues(): Partial<TypeDocOptions>;
    /**
     * Gets a value for the given option key, throwing if the option has not been declared.
     * @param name
     */
    getValue<K extends keyof TypeDocOptions>(name: K): TypeDocOptionValues[K];
    getValue(name: NeverIfInternal<string>): unknown;
    /**
     * Gets the set compiler options.
     */
    getCompilerOptions(): ts.CompilerOptions;
    /**
     * Sets the given declared option. Throws if setting the option fails.
     * @param name
     * @param value
     */
    setValue<K extends keyof TypeDocAndTSOptions>(name: K, value: TypeDocAndTSOptions[K]): void;
    setValue(name: NeverIfInternal<string>, value: NeverIfInternal<unknown>): void;
    /**
     * Sets all the given option values, throws if any value fails to be set.
     * @param obj
     * @deprecated will be removed in 0.19. Use setValue in a loop instead.
     */
    setValues(obj: NeverIfInternal<Partial<TypeDocAndTSOptions>>): void;
    /**
     * Sets the value of a given option to its default value.
     * @param declaration The option whose value should be reset.
     */
    private setOptionValueToDefault;
}
/**
 * Binds an option to the given property. Does not register the option.
 *
 * @since v0.16.3
 */
export declare function BindOption<K extends keyof TypeDocOptionMap>(name: K): <IK extends PropertyKey>(target: ({
    application: Application;
} | {
    options: Options;
}) & {
    [K2 in IK]: TypeDocOptionValues[K];
}, key: IK) => void;
/**
 * Binds an option to the given property. Does not register the option.
 * @since v0.16.3
 *
 * @privateRemarks
 * This overload is intended for plugin use only with looser type checks. Do not use internally.
 */
export declare function BindOption(name: NeverIfInternal<string>): (target: {
    application: Application;
} | {
    options: Options;
}, key: PropertyKey) => void;
