"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BindOption = exports.Options = void 0;
const declaration_1 = require("./declaration");
const sources_1 = require("./sources");
const __1 = require("..");
const help_1 = require("./help");
/**
 * Maintains a collection of option declarations split into TypeDoc options
 * and TypeScript options. Ensures options are of the correct type for calling
 * code.
 *
 * ### Case Sensitivity
 * All option keys are case insensitive. The following lines will each get the same value.
 * ```ts
 * const x = options.getValue('name');
 * const y = options.getValue('NAME');
 * const z = options.tryGetValue('NaMe').unwrap();
 * ```
 *
 * **WARNING**: This case insensitivity is primarily intended to ease command line use and for
 * backward compatibility. It may change in a future release. Any code using TypeDoc options
 * programmatically should conform to the case indicated in the [[TypeDocOptions]] interface.
 *
 * ### Option Discovery
 *
 * Since plugins commonly add custom options, and TypeDoc does not permit options which have
 * not been declared to be set, options must be read twice. The first time options are read,
 * a noop logger is passed so that any errors are ignored. Then, after loading plugins, options
 * are read again, this time with the logger specified by the application.
 *
 * Options are read in a specific order.
 * 1. argv (0) - Must be read first since it should change the files read when
 *    passing --options or --tsconfig.
 * 2. typedoc-json (100) - Read next so that it can specify the tsconfig.json file to read.
 * 3. tsconfig-json (200) - Last config file reader, cannot specify the typedoc.json file to read.
 * 4. argv (300) - Read argv again since any options set there should override those set in config
 *    files.
 */
class Options {
    constructor(logger) {
        this._readers = [];
        this._declarations = new Map();
        // Converted values, would be TypeDocOptions, but TS doesn't deal with that well.
        this._values = {};
        this._compilerOptions = {};
        this._logger = logger;
    }
    /**
     * Sets the logger used when an option declaration fails to be added.
     * @param logger
     */
    setLogger(logger) {
        this._logger = logger;
    }
    /**
     * Adds the option declarations declared by the TypeDoc and all supported TypeScript declarations.
     */
    addDefaultDeclarations() {
        sources_1.addTSOptions(this);
        sources_1.addTypeDocOptions(this);
    }
    /**
     * Gets a help string to display to the user if the -h flag is passed.
     */
    getHelp() {
        return help_1.getOptionsHelp(this);
    }
    /**
     * Resets the option bag to all default values.
     */
    reset() {
        for (const declaration of this._declarations.values()) {
            this.setOptionValueToDefault(declaration);
        }
        this._compilerOptions = {};
    }
    /**
     * Adds an option reader that will be used to read configuration values
     * from the command line, configuration files, or other locations.
     * @param reader
     */
    addReader(reader) {
        __1.insertOrderSorted(this._readers, reader);
    }
    /**
     * Removes all readers of a given name.
     * @param name
     */
    removeReaderByName(name) {
        this._readers = this._readers.filter((reader) => reader.name !== name);
    }
    read(logger) {
        for (const reader of this._readers) {
            reader.read(this, logger);
        }
    }
    addDeclaration(declaration) {
        const names = [declaration.name];
        if (declaration.short) {
            names.push(declaration.short);
        }
        for (const name of names) {
            // Check for registering the same declaration twice, should not be an error.
            const decl = this.getDeclaration(name);
            if (decl && decl !== declaration) {
                this._logger.error(`The option ${name} has already been registered`);
            }
            else {
                this._declarations.set(name.toLowerCase(), declaration);
            }
        }
        this.setOptionValueToDefault(declaration);
    }
    /**
     * Adds the given declarations to the container
     * @param declarations
     *
     * @privateRemarks
     * This function explicitly provides a way out of the strict typing enforced
     * by {@link addDeclaration}. It should only be used with careful validation
     * of the declaration map. Internally, it is only used for adding TS options
     */
    addDeclarations(declarations) {
        for (const decl of declarations) {
            this.addDeclaration(decl);
        }
    }
    /**
     * Removes a declared option.
     * WARNING: This is probably a bad idea. If you do this you will probably cause a crash
     * when code assumes that an option that it declared still exists.
     * @param name
     */
    removeDeclarationByName(name) {
        const declaration = this.getDeclaration(name);
        if (declaration) {
            this._declarations.delete(declaration.name.toLowerCase());
            if (declaration.short) {
                this._declarations.delete(declaration.short.toLowerCase());
            }
            delete this._values[declaration.name];
        }
    }
    /**
     * Gets a declaration by either its name or short name.
     * @param name
     */
    getDeclaration(name) {
        return this._declarations.get(name.toLowerCase());
    }
    /**
     * Gets all declarations in the options with a given scope.
     * @param scope
     */
    getDeclarationsByScope(scope) {
        return __1.uniq(Array.from(this._declarations.values())).filter((declaration) => { var _a; return ((_a = declaration.scope) !== null && _a !== void 0 ? _a : declaration_1.ParameterScope.TypeDoc) === scope; });
    }
    isDefault(name) {
        var _a;
        // getValue will throw if the declaration does not exist.
        return (this.getValue(name) === ((_a = this.getDeclaration(name)) === null || _a === void 0 ? void 0 : _a.defaultValue));
    }
    /**
     * Gets all of the TypeDoc option values defined in this option container.
     * Note that this is a *shallow* clone of the internally defined options. If a plugin defines
     * object options which contain mutable properties, it is their responsibility to correctly
     * handle mutability.
     */
    getRawValues() {
        return { ...this._values };
    }
    getValue(name) {
        const declaration = this.getDeclaration(name);
        if (!declaration) {
            throw new Error(`Unknown option '${name}'`);
        }
        if (declaration.scope === declaration_1.ParameterScope.TypeScript) {
            throw new Error("TypeScript options must be fetched with getCompilerOptions.");
        }
        return this._values[declaration.name];
    }
    /**
     * Gets the set compiler options.
     */
    getCompilerOptions() {
        return { ...this._compilerOptions };
    }
    setValue(name, value) {
        const declaration = this.getDeclaration(name);
        if (!declaration) {
            throw new Error(`Tried to set an option (${name}) that was not declared.`);
        }
        const converted = declaration_1.convert(value, declaration);
        const bag = declaration.scope === declaration_1.ParameterScope.TypeScript
            ? this._compilerOptions
            : this._values;
        bag[declaration.name] = converted;
    }
    /**
     * Sets all the given option values, throws if any value fails to be set.
     * @param obj
     * @deprecated will be removed in 0.19. Use setValue in a loop instead.
     */
    setValues(obj) {
        this._logger.warn("Options.setValues is deprecated and will be removed in 0.19.");
        for (const [name, value] of Object.entries(obj)) {
            this.setValue(name, value);
        }
    }
    /**
     * Sets the value of a given option to its default value.
     * @param declaration The option whose value should be reset.
     */
    setOptionValueToDefault(declaration) {
        if (declaration.scope !== declaration_1.ParameterScope.TypeScript) {
            // No need to convert the defaultValue for a map type as it has to be of a specific type
            if (declaration.type === declaration_1.ParameterType.Map) {
                this._values[declaration.name] = declaration.defaultValue;
            }
            else if (declaration.type === declaration_1.ParameterType.Number) {
                // Don't use convert for number options to allow every possible number as a default value
                this._values[declaration.name] = declaration.defaultValue || 0;
            }
            else {
                this._values[declaration.name] = declaration_1.convert(declaration.defaultValue, declaration);
            }
        }
    }
}
exports.Options = Options;
function BindOption(name) {
    return function (target, key) {
        Object.defineProperty(target, key, {
            get() {
                if ("options" in this) {
                    return this.options.getValue(name);
                }
                else {
                    return this.application.options.getValue(name);
                }
            },
            enumerable: true,
            configurable: true,
        });
    };
}
exports.BindOption = BindOption;
//# sourceMappingURL=options.js.map