"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadPlugins = void 0;
const fs = require("fs");
const path_1 = require("path");
function loadPlugins(app, plugins, logger) {
    if (plugins.some((plugin) => plugin.toLowerCase() === "none")) {
        return;
    }
    if (plugins.length === 0) {
        plugins = discoverNpmPlugins(logger);
    }
    else {
        plugins = resolvePluginPaths(plugins);
    }
    for (const plugin of plugins) {
        loadPlugin(app, plugin, logger);
    }
}
exports.loadPlugins = loadPlugins;
function loadPlugin(app, plugin, logger) {
    try {
        // We can't use import here, we're dynamically importing things...
        // eslint-disable-next-line
        const instance = require(plugin);
        if (typeof instance.load === "function") {
            instance.load(app);
            logger.write("Loaded plugin %s", plugin);
        }
        else {
            logger.error("Invalid structure in plugin %s, no load function found.", plugin);
        }
    }
    catch (error) {
        logger.error("The plugin %s could not be loaded.", plugin);
        logger.verbose(error.stack);
    }
}
function discoverNpmPlugins(logger) {
    const result = [];
    discover();
    return result;
    /**
     * Find all parent folders containing a `node_modules` subdirectory.
     */
    function discover() {
        let path = process.cwd(), previous;
        do {
            const modules = path_1.join(path, "node_modules");
            if (fs.existsSync(modules) && fs.statSync(modules).isDirectory()) {
                discoverModules(modules);
            }
            previous = path;
            path = path_1.resolve(path_1.join(previous, ".."));
        } while (previous !== path);
    }
    /**
     * Scan the given `node_modules` directory for TypeDoc plugins.
     */
    function discoverModules(basePath) {
        const candidates = [];
        fs.readdirSync(basePath).forEach((name) => {
            const dir = path_1.join(basePath, name);
            if (name.startsWith("@") && fs.statSync(dir).isDirectory()) {
                fs.readdirSync(dir).forEach((n) => {
                    candidates.push(path_1.join(name, n));
                });
            }
            candidates.push(name);
        });
        candidates.forEach((name) => {
            const infoFile = path_1.join(basePath, name, "package.json");
            if (!fs.existsSync(infoFile)) {
                return;
            }
            const info = loadPackageInfo(infoFile);
            if (isPlugin(info)) {
                result.push(path_1.join(basePath, name));
            }
        });
    }
    /**
     * Load and parse the given `package.json`.
     */
    function loadPackageInfo(fileName) {
        try {
            // Note: Use `require` here instead of read + JSON.parse since the latter
            // opens up BOM issues. See GH#1192
            return require(fileName);
        }
        catch (error) {
            logger.error("Could not parse %s", fileName);
            return {};
        }
    }
    /**
     * Test whether the given package info describes a TypeDoc plugin.
     */
    function isPlugin(info) {
        const keywords = info.keywords;
        if (!keywords || !Array.isArray(keywords)) {
            return false;
        }
        return keywords.some((keyword) => typeof keyword === "string" && keyword.toLowerCase() === "typedocplugin");
    }
}
function resolvePluginPaths(plugins) {
    const cwd = process.cwd();
    return plugins.map((plugin) => {
        // treat plugins that start with `.` as relative, requiring resolution
        if (plugin.startsWith(".")) {
            return path_1.resolve(cwd, plugin);
        }
        return plugin;
    });
}
//# sourceMappingURL=plugins.js.map